#-*- coding: utf-8 -*-
from django.template import Library
from django.utils import six
import re

register = Library()

RE_SIZE = re.compile(r'(\d+)x(\d+)$')


def _recalculate_size(size, index, divisor=0, padding=0,
                      keep_aspect_ratio=False):
    new_one = size[index]
    if divisor:
        new_one = new_one / float(divisor)
    if padding:
        new_one = new_one - padding
    if keep_aspect_ratio:
        new_two = int(float(new_one) * \
                      float(size[int(not index)]) / size[index])
    else:
        new_two = int(size[int(not index)])

    new_one = int(new_one)
    if index:
        return (new_two, new_one)
    return new_one, new_two


def _resize(original_size, index, divisor=0, padding=0,
            keep_aspect_ratio=False):
    if isinstance(original_size, six.text_type):
        m = RE_SIZE.match(original_size)
        if m:
            original_size = (int(m.group(1)), int(m.group(2)))
        else:
            return original_size
    else:
        try:
            original_size = (int(original_size[0]), int(original_size[1]))
        except (TypeError, ValueError):
            return original_size
    try:
        padding = int(padding)
        divisor = int(divisor)
    except (TypeError, ValueError):
        return original_size
    # Re-calculate size
    new_x, new_y = _recalculate_size(original_size, index, divisor=divisor,
                                     padding=padding,
                                     keep_aspect_ratio=keep_aspect_ratio)
    return (new_x, new_y)


def extra_padding_x(original_size, padding):
    """
    Reduce the width of `original_size` by `padding`
    """
    return _resize(original_size, 0, padding=padding)
extra_padding_x = register.filter(extra_padding_x)


def extra_padding_x_keep_ratio(original_size, padding):
    """
    Reduce the width of `original_size` by `padding`, maintaining the aspect
    ratio.
    """
    return _resize(original_size, 0, padding=padding, keep_aspect_ratio=True)
extra_padding_x_keep_ratio = register.filter(extra_padding_x_keep_ratio)


def extra_padding_y(original_size, padding):
    """
    Reduce the height of `original_size` by `padding`
    """
    return _resize(original_size, 1, padding=padding)
extra_padding_y = register.filter(extra_padding_y)


def extra_padding_y_keep_ratio(original_size, padding):
    """
    Reduce the height of `original_size` by `padding`, maintaining the aspect
    ratio.
    """
    return _resize(original_size, 1, padding=padding, keep_aspect_ratio=True)
extra_padding_y_keep_ratio = register.filter(extra_padding_y_keep_ratio)


def divide_x_by(original_size, divisor):
    return _resize(original_size, 0, divisor=divisor)
devide_x_by = register.filter(divide_x_by)


def divide_y_by(original_size, divisor):
    return _resize(original_size, 1, divisor=divisor)
devide_y_by = register.filter(divide_y_by)


def divide_xy_by(original_size, divisor):
    size = divide_x_by(original_size, divisor=divisor)
    size = divide_y_by(size, divisor=divisor)
    return size
divide_xy_by = register.filter(divide_xy_by)



from django import forms
from django_select2.forms import ModelSelect2Widget
from leonardo.module.media.models import File

FILE_SEARCH_FIELDS = [
    'original_filename__icontains',
    'name__icontains',
    'description__icontains',
]

FOLDER_SEARCH_FIELDS = [
    'name__icontains',
]


class FileSelectWidget(ModelSelect2Widget):

    model = File

    search_fields = FILE_SEARCH_FIELDS


class FileFieldMixin(object):

    def __init__(self, *args, **kwargs):
        super(FileFieldMixin, self).__init__(
            queryset=getattr(self, 'model', File).objects.all(),
            empty_label='---',
            widget=FileSelectWidget(), *args, **kwargs)

    def label_from_instance(self, obj):
        """
        Coerces ``value`` to a Python data type.
        """
        return obj.pretty_logical_path


class FileField(FileFieldMixin, forms.ModelChoiceField):
    '''Basic File Field for easy selecting files anywhere'''
    pass


class FileMultipleField(FileFieldMixin, forms.ModelMultipleChoiceField):
    '''Multiple File Field with select widget'''
    pass



#-*- coding: utf-8 -*-
import logging
import warnings

from django import forms
from django.contrib.admin.sites import site
from django.contrib.admin.widgets import ForeignKeyRawIdWidget
from django.core.urlresolvers import reverse
from django.db import models
from django.template.loader import render_to_string
from django.utils.safestring import mark_safe
from filer.utils.compatibility import truncate_words

from .. import settings as filer_settings
from ..models import File, Video, Vector, Document
from .utils import FileField, FileMultipleField
logger = logging.getLogger(__name__)


class AdminFileWidget(ForeignKeyRawIdWidget):
    choices = None

    def render(self, name, value, attrs=None):
        obj = self.obj_for_value(value)
        css_id = attrs.get('id', 'id_image_x')
        css_id_thumbnail_img = "%s_thumbnail_img" % css_id
        css_id_description_txt = "%s_description_txt" % css_id
        related_url = None
        if value:
            try:
                file_obj = File.objects.get(pk=value)
                related_url = file_obj.logical_folder.\
                    get_admin_directory_listing_url_path()
            except Exception as e:
                # catch exception and manage it. We can re-raise it for debugging
                # purposes and/or just logging it, provided user configured
                # proper logging configuration
                if filer_settings.FILER_ENABLE_LOGGING:
                    logger.error('Error while rendering file widget: %s', e)
                if filer_settings.FILER_DEBUG:
                    raise
        if not related_url:
            related_url = reverse('admin:filer-directory_listing-last')
        params = self.url_parameters()
        if params:
            lookup_url = '?' + '&amp;'.join(
                ['%s=%s' % (k, v) for k, v in list(params.items())])
        else:
            lookup_url = ''
        if not 'class' in attrs:
            # The JavaScript looks for this hook.
            attrs['class'] = 'vForeignKeyRawIdAdminField'
        # rendering the super for ForeignKeyRawIdWidget on purpose here because
        # we only need the input and none of the other stuff that
        # ForeignKeyRawIdWidget adds
        hidden_input = super(ForeignKeyRawIdWidget, self).render(
            name, value, attrs)
        filer_static_prefix = filer_settings.FILER_STATICMEDIA_PREFIX
        if not filer_static_prefix[-1] == '/':
            filer_static_prefix += '/'
        context = {
            'hidden_input': hidden_input,
            'lookup_url': '%s%s' % (related_url, lookup_url),
            'thumb_id': css_id_thumbnail_img,
            'span_id': css_id_description_txt,
            'object': obj,
            'lookup_name': name,
            'filer_static_prefix': filer_static_prefix,
            'clear_id': '%s_clear' % css_id,
            'id': css_id,
        }
        html = render_to_string('admin/media/widgets/admin_file.html', context)
        return mark_safe(html)

    def label_for_value(self, value):
        obj = self.obj_for_value(value)
        return '&nbsp;<strong>%s</strong>' % truncate_words(obj, 14)

    def obj_for_value(self, value):
        try:
            key = self.rel.get_related_field().name
            obj = self.rel.to._default_manager.get(**{key: value})
        except:
            obj = None
        return obj

    class Media:
        js = (
            filer_settings.FILER_STATICMEDIA_PREFIX + 'js/popup_handling.js',
            filer_settings.FILER_STATICMEDIA_PREFIX + 'js/widget.js',
        )


class AdminFileFormField(forms.ModelChoiceField):
    widget = AdminFileWidget

    def __init__(self, rel, queryset, to_field_name, *args, **kwargs):
        self.rel = rel
        self.queryset = queryset
        self.to_field_name = to_field_name
        self.max_value = None
        self.min_value = None
        kwargs.pop('widget', None)
        super(AdminFileFormField, self).__init__(
            queryset, widget=self.widget(rel, site), *args, **kwargs)

    def widget_attrs(self, widget):
        widget.required = self.required
        return {}


class FilerFileField(models.ForeignKey):
    default_form_class = AdminFileFormField
    default_model_class = File

    def __init__(self, **kwargs):
        # We hard-code the `to` argument for ForeignKey.__init__
        if "to" in kwargs.keys():
            old_to = kwargs.pop("to")
            msg = "%s can only be a ForeignKey to %s; %s passed" % (
                self.__class__.__name__, self.default_model_class.__name__, old_to
            )
            warnings.warn(msg, SyntaxWarning)
        kwargs['to'] = self.default_model_class
        return super(FilerFileField, self).__init__(**kwargs)

    def formfield(self, **kwargs):
        # This is a fairly standard way to set up some defaults
        # while letting the caller override them.
        defaults = {
            'form_class': self.default_form_class,
            'rel': self.rel,
        }
        defaults.update(kwargs)
        return super(FilerFileField, self).formfield(**defaults)

    def south_field_triple(self):
        "Returns a suitable description of this field for South."
        # We'll just introspect ourselves, since we inherit.
        from south.modelsinspector import introspector
        field_class = "django.db.models.fields.related.ForeignKey"
        args, kwargs = introspector(self)
        # That's our definition!
        return (field_class, args, kwargs)


class FileField(FileField):
    model = File


class VideoField(FileField):
    model = Video


class VectorField(FileField):
    model = Vector


class DocumentField(FileField):
    model = Document


class MultipleFileField(FileMultipleField):
    model = File


class MultipleDocumentField(FileMultipleField):
    model = Document


class MultipleVectorField(FileMultipleField):
    model = Vector


class MultipleVideoField(FileMultipleField):
    model = Video



import xml.parsers.expat
from decimal import Decimal

from django.conf import settings
from django.template.loader import render_to_string
import xml.dom.minidom

import pprint
pp = pprint.PrettyPrinter(indent=1, width=20).pprint


def autofill_svg_size(sender, **kwargs):
    if not kwargs['instance'].autofill_dimensions:
        return
    
    dims = {}
    def start_element(name, attrs):
        if name == 'svg':
            if u'height' in attrs and u'width' in attrs:
                dims['height'] = attrs[u'height']
                dims['width'] = attrs[u'width']

                
    p = xml.parsers.expat.ParserCreate()    
    p.StartElementHandler = start_element
    p.Parse(kwargs['instance'].file.read())
    if 'height' in dims and 'width' in dims:
        #attention! measure unit is deleted! need to find - what measure units can be used for size
        #and how it can be implemented in svg-editor
        kwargs['instance'].height = Decimal(dims['height'].rstrip('px').rstrip('em').rstrip('%'))
        kwargs['instance'].width = Decimal(dims['width'].rstrip('px').rstrip('em').rstrip('%'))



class recompose(object):
    def __init__(self, modify_type):
        self.modify_type = modify_type
        self.sizes = []
        self.urls = []
    
    def __call__(self, *args, **kwargs):
        self.sizes = []
        self.urls = []
        content_tokens = []
        content_attr_name = kwargs['sender'].__name__.split("_")[-1]
        modified_contents = getattr(kwargs['instance'],
                           content_attr_name).filter(id__in=list(kwargs['pk_set']))
        for modified_content in modified_contents: 
            content_tokens += {"orm_id":unicode(modified_content.id),
                               "orm_page_id":unicode(modified_content.parent.id),
                               "orm_region":unicode(modified_content.region),
                               "orm_ordering":unicode(modified_content.ordering)},
            self.sizes += (modified_content.markup.height, 
                           modified_content.markup.width),
            self.urls += modified_content.markup.file.url,
        return self.modify(markup_path=kwargs['instance'].markup.file.path,
                           content_tokens=content_tokens,
                           modify_type=self.modify_type,
                           sizes =self.sizes,)
        
    def modify(self, markup_path="", content_tokens=[], sizes=[], modify_type=""):
        
        def marked_elements(root, element_name, content_tokens):
            for element in root.getElementsByTagName(element_name):
                attrs = dict(((attr_node.name, attr_node.value) for\
                            attr_node in element._attrs.values()))
                for content_token in content_tokens:
                    related_attrs = dict((a[0],a[1]) for a in attrs.items() if \
                                          a[0] in content_token \
                                          and a[1] == content_token[a[0]])
                    if len(related_attrs) == len(content_token):
                        yield element
                        

        def adder(doc):
            if not len(list(marked_elements(doc, 'image', content_tokens))):
                for content_token in content_tokens:
#                    print content_tokens.index(content_token)
#                    print self.urls
#                    print self.urls[content_tokens.index(content_token)]
                    image_node = xml.dom.minidom.parseString(render_to_string('svg/image_node.xml',
                                                                           {'token':content_token,
                                                                            'size':self.sizes[content_tokens.index(content_token)],
                                                                            'url':self.urls[content_tokens.index(content_token)]})).\
                                                                            childNodes[0].childNodes[0]
                    image_node.ownerDocument = None
                    image_node.parentNode = None
                    doc.childNodes[0].appendChild(image_node)
        def remover(doc):
            for element in marked_elements(doc, 'image', content_tokens):
                element.parentNode.removeChild(element)
        
        if modify_type == 'add':
            modifier = adder
        elif modify_type == 'remove':
            modifier = remover
        else:
            raise ValueError("'modify_type' must be either 'add' or 'remove' string")
        markup = open(markup_path)
        doc = xml.dom.minidom.parse(markup)
        markup.close()
        #markup = open(markup_path, mode='w')
        modifier(doc)
        markup = file(markup_path, mode='w')
        markup.write(doc.toprettyxml())
        markup.close()


RECOMPOSE_MAP = {('pre_remove', "SVGComposerContentType_composers"):recompose('remove'),
                 ('post_add', "SVGComposerContentType_composers"):recompose('add'),
                 ('pre_remove', "SVGComposerContentType_components"):recompose('remove'),
                 ('post_add', "SVGComposerContentType_components"):recompose('add'),}


def recompose_dispatcher(*args, **kwargs):
    if kwargs['instance'].__class__.__name__ == 'SVGComposerContentType':
        if  (kwargs['action'],kwargs['sender'].__name__) in RECOMPOSE_MAP:
                RECOMPOSE_MAP[(kwargs['action'],kwargs['sender'].__name__)](*args, **kwargs)
                
def create_markup_if_empty(*args, **kwargs):
    
    if kwargs['sender'].__name__ == 'SVGComposerContentType' and\
       not kwargs['instance'].markup and\
       not hasattr(kwargs['instance'],'_markup_resetted'):
        print "reset"    
        kwargs['instance'].reset_markup()
        setattr(kwargs['instance'],'_markup_resetted', True)
        kwargs['instance'].save()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import filer.fields.multistorage_file
import filer.models.mixins
import leonardo.module.media.models
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        ('auth', '0001_initial'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('contenttypes', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Clipboard',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
            ],
            options={
                'verbose_name': 'clipboard',
                'verbose_name_plural': 'clipboards',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='ClipboardItem',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('clipboard', models.ForeignKey(verbose_name='clipboard', to='media.Clipboard')),
            ],
            options={
                'verbose_name': 'clipboard item',
                'verbose_name_plural': 'clipboard items',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='DocumentTranslation',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('language_code', models.CharField(default=b'en', max_length=10, verbose_name='language', choices=[(b'en', b'EN'), (b'cs', b'CS')])),
            ],
            options={
                'verbose_name': 'document translation',
                'verbose_name_plural': 'document translations',
            },
            bases=(models.Model, leonardo.module.media.models.MediaTranslationMixin),
        ),
        migrations.CreateModel(
            name='File',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('file', filer.fields.multistorage_file.MultiStorageFileField(max_length=255, upload_to=filer.fields.multistorage_file.generate_filename_multistorage, null=True, verbose_name='file', blank=True)),
                ('_file_size', models.IntegerField(null=True, verbose_name='file size', blank=True)),
                ('sha1', models.CharField(default='', max_length=40, verbose_name='sha1', blank=True)),
                ('has_all_mandatory_data', models.BooleanField(default=False, verbose_name='has all mandatory data', editable=False)),
                ('original_filename', models.CharField(max_length=255, null=True, verbose_name='original filename', blank=True)),
                ('name', models.CharField(default='', max_length=255, verbose_name='name', blank=True)),
                ('description', models.TextField(null=True, verbose_name='description', blank=True)),
                ('uploaded_at', models.DateTimeField(auto_now_add=True, verbose_name='uploaded at')),
                ('modified_at', models.DateTimeField(auto_now=True, verbose_name='modified at')),
                ('is_public', models.BooleanField(default=True, help_text='Disable any permission checking for this file. File will be publicly accessible to anyone.', verbose_name='Permissions disabled')),
            ],
            options={
                'verbose_name': 'file',
                'verbose_name_plural': 'files',
            },
            bases=(models.Model, filer.models.mixins.IconsMixin),
        ),
        migrations.CreateModel(
            name='Document',
            fields=[
                ('file_ptr', models.OneToOneField(parent_link=True, auto_created=True, primary_key=True, serialize=False, to='media.File')),
            ],
            options={
                'verbose_name': 'document',
                'verbose_name_plural': 'documents',
            },
            bases=(leonardo.module.media.models.MediaMixin, 'media.file'),
        ),
        migrations.CreateModel(
            name='Flash',
            fields=[
                ('file_ptr', models.OneToOneField(parent_link=True, auto_created=True, primary_key=True, serialize=False, to='media.File')),
            ],
            options={
                'verbose_name': 'flash video',
                'verbose_name_plural': 'flash videos',
            },
            bases=(leonardo.module.media.models.MediaMixin, 'media.file'),
        ),
        migrations.CreateModel(
            name='Folder',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=255, verbose_name='name')),
                ('uploaded_at', models.DateTimeField(auto_now_add=True, verbose_name='uploaded at')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='created at')),
                ('modified_at', models.DateTimeField(auto_now=True, verbose_name='modified at')),
                ('lft', models.PositiveIntegerField(editable=False, db_index=True)),
                ('rght', models.PositiveIntegerField(editable=False, db_index=True)),
                ('tree_id', models.PositiveIntegerField(editable=False, db_index=True)),
                ('level', models.PositiveIntegerField(editable=False, db_index=True)),
                ('owner', models.ForeignKey(related_name='filer_owned_folders', verbose_name='owner', blank=True, to=settings.AUTH_USER_MODEL, null=True)),
                ('parent', models.ForeignKey(related_name='media_folder_children', verbose_name='parent', blank=True, to='media.Folder', null=True)),
            ],
            options={
                'ordering': ('name',),
                'verbose_name': 'Folder',
                'verbose_name_plural': 'Folders',
                'permissions': (('can_use_Folder_listing', 'Can use Folder listing'),),
            },
            bases=(models.Model, filer.models.mixins.IconsMixin),
        ),
        migrations.CreateModel(
            name='FolderPermission',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('type', models.SmallIntegerField(default=0, verbose_name='type', choices=[(0, 'all items'), (1, 'this item only'), (2, 'this item and all children')])),
                ('everybody', models.BooleanField(default=False, verbose_name='everybody')),
                ('can_edit', models.SmallIntegerField(default=None, null=True, verbose_name='can edit', blank=True, choices=[(1, 'allow'), (0, 'deny')])),
                ('can_read', models.SmallIntegerField(default=None, null=True, verbose_name='can read', blank=True, choices=[(1, 'allow'), (0, 'deny')])),
                ('can_add_children', models.SmallIntegerField(default=None, null=True, verbose_name='can add children', blank=True, choices=[(1, 'allow'), (0, 'deny')])),
                ('folder', models.ForeignKey(verbose_name='folder', blank=True, to='media.Folder', null=True)),
                ('group', models.ForeignKey(related_name='folder_permissions', verbose_name='group', blank=True, to='auth.Group', null=True)),
                ('user', models.ForeignKey(related_name='folder_permissions', verbose_name='user', blank=True, to=settings.AUTH_USER_MODEL, null=True)),
            ],
            options={
                'verbose_name': 'folder permission',
                'verbose_name_plural': 'folder permissions',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='FolderTranslation',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('language_code', models.CharField(default=b'en', max_length=10, verbose_name='language', choices=[(b'en', b'EN'), (b'cs', b'CS')])),
                ('name', models.CharField(default=b'', max_length=255, verbose_name='name', blank=True)),
                ('parent', models.ForeignKey(related_name='translations', to='media.Folder')),
            ],
            options={
                'verbose_name': 'Folder translation',
                'verbose_name_plural': 'Folder translations',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Image',
            fields=[
                ('file_ptr', models.OneToOneField(parent_link=True, auto_created=True, primary_key=True, serialize=False, to='media.File')),
                ('_height', models.IntegerField(null=True, blank=True)),
                ('_width', models.IntegerField(null=True, blank=True)),
                ('default_alt_text', models.CharField(max_length=255, null=True, verbose_name='default alt text', blank=True)),
                ('default_caption', models.CharField(max_length=255, null=True, verbose_name='default caption', blank=True)),
                ('subject_location', models.CharField(default=None, max_length=64, null=True, verbose_name='subject location', blank=True)),
                ('date_taken', models.DateTimeField(verbose_name='date taken', null=True, editable=False, blank=True)),
                ('author', models.CharField(max_length=255, null=True, verbose_name='author', blank=True)),
                ('must_always_publish_author_credit', models.BooleanField(default=False, verbose_name='must always publish author credit')),
                ('must_always_publish_copyright', models.BooleanField(default=False, verbose_name='must always publish copyright')),
            ],
            options={
                'verbose_name': 'image',
                'verbose_name_plural': 'images',
            },
            bases=('media.file',),
        ),
        migrations.CreateModel(
            name='ImageTranslation',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('language_code', models.CharField(default=b'en', max_length=10, verbose_name='language', choices=[(b'en', b'EN'), (b'cs', b'CS')])),
                ('name', models.CharField(default=b'', max_length=255, verbose_name='name', blank=True)),
                ('default_alt_text', models.CharField(default=b'', max_length=255, verbose_name='default alt text', blank=True)),
                ('default_caption', models.CharField(default=b'', max_length=255, verbose_name='default caption', blank=True)),
                ('description', models.TextField(null=True, verbose_name='description', blank=True)),
                ('parent', models.ForeignKey(related_name='translations', to='media.Image')),
            ],
            options={
                'verbose_name': 'image translation',
                'verbose_name_plural': 'image translations',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Vector',
            fields=[
                ('file_ptr', models.OneToOneField(parent_link=True, auto_created=True, primary_key=True, serialize=False, to='media.File')),
            ],
            options={
                'verbose_name': 'vector',
                'verbose_name_plural': 'vetors',
            },
            bases=(leonardo.module.media.models.MediaMixin, 'media.file'),
        ),
        migrations.CreateModel(
            name='VectorTranslation',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('language_code', models.CharField(default=b'en', max_length=10, verbose_name='language', choices=[(b'en', b'EN'), (b'cs', b'CS')])),
                ('parent', models.ForeignKey(related_name='translations', to='media.Vector')),
            ],
            options={
                'verbose_name': 'vector translation',
                'verbose_name_plural': 'vector translations',
            },
            bases=(models.Model, leonardo.module.media.models.MediaTranslationMixin),
        ),
        migrations.CreateModel(
            name='Video',
            fields=[
                ('file_ptr', models.OneToOneField(parent_link=True, auto_created=True, primary_key=True, serialize=False, to='media.File')),
            ],
            options={
                'verbose_name': 'video',
                'verbose_name_plural': 'videos',
            },
            bases=(leonardo.module.media.models.MediaMixin, 'media.file'),
        ),
        migrations.CreateModel(
            name='VideoTranslation',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('language_code', models.CharField(default=b'en', max_length=10, verbose_name='language', choices=[(b'en', b'EN'), (b'cs', b'CS')])),
                ('parent', models.ForeignKey(related_name='translations', to='media.Video')),
            ],
            options={
                'verbose_name': 'video translation',
                'verbose_name_plural': 'video translations',
            },
            bases=(models.Model, leonardo.module.media.models.MediaTranslationMixin),
        ),
        migrations.AlterUniqueTogether(
            name='folder',
            unique_together=set([('parent', 'name')]),
        ),
        migrations.AddField(
            model_name='file',
            name='folder',
            field=models.ForeignKey(related_name='media_file_files', verbose_name='folder', blank=True, to='media.Folder', null=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='file',
            name='owner',
            field=models.ForeignKey(related_name='owned_files', verbose_name='owner', blank=True, to=settings.AUTH_USER_MODEL, null=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='file',
            name='polymorphic_ctype',
            field=models.ForeignKey(related_name='polymorphic_media.file_set+', editable=False, to='contenttypes.ContentType', null=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='documenttranslation',
            name='parent',
            field=models.ForeignKey(related_name='translations', to='media.Document'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='clipboarditem',
            name='file',
            field=models.ForeignKey(verbose_name='file', to='media.File'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='clipboard',
            name='files',
            field=models.ManyToManyField(related_name='in_clipboards', verbose_name='files', through='media.ClipboardItem', to='media.File'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='clipboard',
            name='user',
            field=models.ForeignKey(related_name='filer_clipboards', verbose_name='user', to=settings.AUTH_USER_MODEL),
            preserve_default=True,
        ),
    ]



# -#- coding: utf-8 -#-

from django.conf import settings
from django.db import models
from django.utils.translation import ugettext_lazy as _
from leonardo.module.media.fields.folder import FolderField
from leonardo.module.web.models import ListWidget
from leonardo.module.web.widgets.forms import WidgetUpdateForm

DETAIL_CHOICES = (
    ('open_modal', _('open in modal window')),
    ('open_new_window', _('open new window')),
    ('on_page', _('disply in page')),
)

SIZE_CHOICES = (
    ('small', _('small')),
    ('medium', _('medium')),
    ('large', _('large')),
)


class FolderForm(WidgetUpdateForm):

    folder = FolderField()


class MediaGalleryWidget(ListWidget):

    feincms_item_editor_form = FolderForm

    icon = "fa fa-picture-o"

    folder = models.ForeignKey('media.Folder', verbose_name=_(
        "Directory"), related_name="%(app_label)s_%(class)s_folders")
    size = models.CharField(max_length=255, verbose_name=_(
        "thumbnail size"), choices=SIZE_CHOICES, default='small')
    detail = models.CharField(max_length=255, verbose_name=_(
        "detail view"), choices=DETAIL_CHOICES, default='modal')

    def thumb_geom(self):
        return getattr(settings,
                       'MEDIA_THUMB_%s_GEOM' % self.size.upper(),
                       '96x96')

    def get_items(self):
        return self.folder.media_file_files.all()

    class Meta:
        abstract = True
        verbose_name = _("media gallery")
        verbose_name_plural = _('media galleries')



# -#- coding: utf-8 -#-

from django import forms
from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.template.loader import render_to_string

from leonardo.forms import Layout, Fieldset, Row, FormHelper, Submit, Reset, SelfHandlingModelForm

from leonardo.module.web.models import Widget

TEST_CHOICES = (
    ('para', _("Paragraphs")),
    ('list', _("Lists")),
    ('heading', _("Headings")),
    ('table', _("Tables")),
    ('form', _("Forms")),
    ('button', _("Buttons")),
    ('typo', _("Typography")),
    ('grid', _("Grid")),
    ('icon', _("Icons")),
)


class VisualTestModel(models.Model):
    datetime_simple = models.DateTimeField(
        verbose_name=_("Date taken"), blank=True, null=True, default='para')

    class Meta:
        abstract = True

CHOICES = (
    ("A", "Alpha"),
    ("B", "Bravo"),
    ("C", "Charlie"),
)


class VisualTestForm(SelfHandlingModelForm):
    character_field = forms.CharField(
        label="Character Field", help_text="I am help text", max_length=30, required=True, widget=forms.TextInput())
    url_field = forms.URLField(
        label='URL field', max_length=100, required=True, widget=forms.TextInput())
    textarea_field = forms.CharField(
        label='Textareafield', required=True, widget=forms.Textarea())
#    hidden_field = forms.CharField(label='textarea_field', required=True, widget=forms.HiddenInput())
    file_field = forms.FileField(label="File Field", required=False)
    password_field_1 = forms.CharField(
        label='Password field', max_length=100, widget=forms.PasswordInput())
    password_field_2 = forms.CharField(
        label='Password field', max_length=100, widget=forms.PasswordInput())

    boolean_field = forms.BooleanField(label="Boolean Field", required=False)
    choice_field = forms.ChoiceField(
        label="Choice Field", required=False, choices=CHOICES)
    multiple_choice_field = forms.MultipleChoiceField(
        label="multiple_choice_field", required=False, choices=CHOICES)

    def __init__(self, *args, **kwargs):
        super(VisualTestForm, self).__init__(*args, **kwargs)

        # Add in a class and id
        self.helper.form_id = 'this-form-rocks'
        self.helper.form_class = 'search'
        self.helper.layout = Layout(
            Fieldset('',
                     Row('character_field', 'url_field'),
                     ),
            Fieldset(_('Contact details'),
                     Row('password_field_1', 'password_field_2'),
                     'boolean_field',
                     'choice_field',
                     ),
            Reset('reset', 'reset button'),
            Submit('enter', 'enter some data')
        )

        self._wrap_all()

    class Meta:
        model = VisualTestModel
        exclude = tuple()


class VisualTestWidget(Widget):
    test = models.CharField(max_length=255, verbose_name=_(
        "type of test"), choices=TEST_CHOICES, default="general")

    def test_page(self):
        return "widget/visualtest/_test_%s.html" % self.test

    class Meta:
        abstract = True
        verbose_name = _("visual tester")
        verbose_name_plural = _('visual testers')

    def render_content(self, options):
        if self.test == 'form':
            form = VisualTestForm()
        else:
            form = None

        return render_to_string(self.get_template, {
            'widget': self,
            'request': options['request'],
            'form': form
        })



from __future__ import absolute_import

from celery import shared_task
from django.core import management
from leonardo.decorators import catch_result


@shared_task
@catch_result
def sync_search_indexes():
    management.call_command('rebuild_index', interactive=False)
    return {'result': 'Rebuild index OK'}



# -#- coding: utf-8 -#-

from django.conf import settings
from django.template.loader import render_to_string
from django.utils.translation import ugettext_lazy as _
from leonardo.module.web.models import Widget, Page


class LanguageSelectorWidget(Widget):

    """blueprint on new field ``excludecurrent`` boolean
    """

    class Meta:
        abstract = True
        verbose_name = _("Language switch")
        verbose_name_plural = _("Language switches")

    def render_content(self, options):
        request = options['request']
        try:
            language = request.session['django_language']
        except:
            language = settings.LANGUAGE_CODE
        languages = {}
        for lang in settings.LANGUAGES:
            languages[lang[0]] = {
                'label': lang[1],
                'code': lang[0],
            }
        page = Page.objects.best_match_for_path(request.path, raise404=True)
        suffix = request.path.replace(page.get_absolute_url(), '')

        if page.language == settings.LANGUAGE_CODE:
            languages[page.language]['page'] = page
            languages[page.language]['suffix'] = suffix
            translated_pages = page.translations.all()
            for trans_page in translated_pages:
                languages[trans_page.language]['page'] = trans_page
                languages[trans_page.language]['suffix'] = suffix
        else:
            if page.translation_of:
                main_page = page.translation_of
                languages[main_page.language]['page'] = main_page
                languages[main_page.language]['suffix'] = suffix
                translated_pages = main_page.translations.all()
                for trans_page in translated_pages:
                    languages[trans_page.language]['page'] = trans_page
                    languages[trans_page.language]['suffix'] = suffix

        return render_to_string(self.get_template_name(), {
            'widget': self,
            'page': page,
            'request': request,
            'language': language,
            'languages': languages
        })




from django.conf import settings
from django.conf.urls import include, patterns, url
from django.conf.urls.static import static
from django.contrib.sitemaps.views import sitemap
from django.views.generic.base import RedirectView, TemplateView
from feincms.module.page.sitemap import PageSitemap

from .base import leonardo

__all__ = ['handler400', 'handler403', 'handler404', 'handler500']

urlpatterns = [
    url(r'^contrib/', include('horizon_contrib.urls'),),
    url(r'^select2/', include('django_select2.urls')),
]

if getattr(settings, 'LEONARDO_AUTH', True):
    urlpatterns += patterns('',
                            url(r'^auth/',
                                include('leonardo.module.leonardo_auth.auth_urls')),
                            )

urlpatterns += leonardo.urlpatterns

if getattr(settings, 'HORIZON_ENABLED', True):
    import horizon
    urlpatterns += patterns('',
                            url(r'', include(horizon.urls)),
                            )

# translation
urlpatterns += patterns('',
                        url(r'^i18n/js/(?P<packages>\S+?)/$',
                            'django.views.i18n.javascript_catalog',
                            name='jsi18n'),
                        url(r'^i18n/setlang/$',
                            'django.views.i18n.set_language',
                            name="set_language"),
                        url(r'^i18n/', include('django.conf.urls.i18n'))
                        )
# secure media
urlpatterns += patterns('',
                        url(r'^', include('leonardo.module.media.server.urls'))
                        )

if settings.DEBUG:

    urlpatterns += static(settings.STATIC_URL,
                          document_root=settings.STATIC_ROOT)
    urlpatterns += static(settings.MEDIA_URL,
                          document_root=settings.MEDIA_ROOT)

    try:
        import debug_toolbar
        urlpatterns += patterns('',
                                url(r'^__debug__/', include(debug_toolbar.urls)),
                                )
    except ImportError:
        pass

# feinCMS
urlpatterns += patterns('',
                        url(r'', include('feincms.urls')),
                        )

sitemaps = {
    'pages': PageSitemap,
}

urlpatterns += patterns('',
                        (r'^sitemap\.xml$', sitemap, {'sitemaps': sitemaps}),
                        (r'^favicon\.ico$', RedirectView.as_view(
                            **{'permanent': True, 'url': settings.FAVICON_PATH}),),
                        (r'^robots\.txt$',
                         TemplateView.as_view(template_name='robots.txt')),
                        (r'^crossdomain\.xml$',
                         TemplateView.as_view(template_name='crossdomain.xml')),
                        )

handler400 = getattr(settings, "HANDLER_400",
                     'leonardo.views.defaults.bad_request')
handler403 = getattr(settings, "HANDLER_403",
                     'leonardo.views.defaults.permission_denied')
handler404 = getattr(settings, "HANDLER_404",
                     'leonardo.views.defaults.page_not_found')
handler500 = getattr(settings, "HANDLER_500",
                     'leonardo.views.defaults.server_error')




from __future__ import absolute_import, unicode_literals

from django.test import TestCase

from leonardo.module.web.models import Page
from leonardo.module.web import default


class WebBaseTest(TestCase):
    def test_01_simple_content_type_creation(self):
        for widget in default.widgets:
            self.assertIsNotNone(Page.content_type_for(widget))




default_app_config = 'leonardo.apps.LeonardoConfig'

__import__('pkg_resources').declare_namespace(__name__)


try:
    from leonardo.base import leonardo  # noqa
except ImportError:
    import warnings

    def simple_warn(message, category, filename, lineno, file=None, line=None):
        return '%s: %s' % (category.__name__, message)

    msg = ("Could not import Leonardo dependencies. "
           "This is normal during installation.\n")
    warnings.formatwarning = simple_warn
    warnings.warn(msg, Warning)



#!/usr/bin/env python

import os
import sys

import django
from django.core.management import execute_from_command_line

possible_topdir = os.path.normpath(os.path.join(os.path.abspath(sys.argv[0]),
                                                os.pardir,
                                                os.pardir,
                                                os.pardir))


if os.path.exists(os.path.join(possible_topdir, 'leonardo', '__init__.py')):
    sys.path.insert(0, possible_topdir)
    sys.path.insert(0, os.path.normpath(os.path.join(possible_topdir, os.pardir)))

os.environ['DJANGO_SETTINGS_MODULE'] = 'leonardo.settings'
django.setup()


application = django.core.handlers.wsgi.WSGIHandler()



from django.template import Library

try:
    from sorl.thumbnail.templatetags.thumbnail import thumbnail as sorl_thumb
    SORL = True
except Exception:
    SORL = False

try:
    from easy_thumbnails.templatetags.thumbnail import thumbnail as easy_thumb
    EASY = True
except Exception:
    EASY = False

register = Library()


def thumbnail(parser, token):
    '''
    This template tag supports both syntax for declare thumbanil in template
    '''

    thumb = None

    if SORL:
        try:
            thumb = sorl_thumb(parser, token)
        except Exception:
            thumb = False

    if EASY and not thumb:
        thumb = easy_thumb(parser, token)

    return thumb

register.tag(thumbnail)




from __future__ import unicode_literals

from django.db import models
from django.utils.encoding import python_2_unicode_compatible, smart_text
from django.utils.functional import cached_property
from django.utils.translation import ugettext_lazy as _
from feincms.module.page.models import BasePage as FeinCMSPage

from ..const import *
from ..processors import edit as edit_processors


@python_2_unicode_compatible
class PageDimension(models.Model):

    page = models.ForeignKey('Page', verbose_name=_('Page'))
    size = models.CharField(
        verbose_name="Size", max_length=20, choices=DISPLAY_SIZE_CHOICES, default='md')
    col1_width = models.IntegerField(verbose_name=_("Column 1 width"),
                                     choices=COLUMN_CHOICES, default=DEFAULT_WIDTH)
    col2_width = models.IntegerField(verbose_name=_("Column 2 width"),
                                     choices=COLUMN_CHOICES, default=DEFAULT_WIDTH)
    col3_width = models.IntegerField(verbose_name=_("Column 3 width"),
                                     choices=COLUMN_CHOICES, default=DEFAULT_WIDTH)

    def __str__(self):
        return "{0} - {1}".format(self.page, self.size)

    class Meta:
        verbose_name = _("Page dimension")
        verbose_name_plural = _("Page dimensions")


@python_2_unicode_compatible
class PageTheme(models.Model):

    name = models.CharField(
        verbose_name=_("Name"), max_length=255, null=True, blank=True)
    label = models.CharField(
        verbose_name=_("Title"), max_length=255, null=True, blank=True)
    template = models.ForeignKey(
        'dbtemplates.Template', verbose_name=_('Template'), related_name='page_templates', limit_choices_to={'name__startswith': "base/page/"})
    styles = models.TextField(verbose_name=_('Style'), blank=True)

    def __str__(self):
        return self.label or super(PageTheme, self).__str__()

    class Meta:
        verbose_name = _("Page theme")
        verbose_name_plural = _("Page themes")


@python_2_unicode_compatible
class PageColorScheme(models.Model):

    name = models.CharField(
        verbose_name=_("Name"), max_length=255, null=True, blank=True)
    label = models.CharField(
        verbose_name=_("Title"), max_length=255, null=True, blank=True)
    styles = models.TextField(verbose_name=_('Styles'), blank=True)
    variables = models.TextField(verbose_name=_('variables'), blank=True)
    theme = models.ForeignKey(
        PageTheme, verbose_name=_('Template'), related_name='templates')

    def __str__(self):
        return self.label or super(PageColorScheme, self).__str__()

    class Meta:
        verbose_name = _("Page color scheme")
        verbose_name_plural = _("Page color schemes")


class Page(FeinCMSPage):

    layout = models.CharField(
        verbose_name=_("Layout"), max_length=25, default='fixed', choices=PAGE_LAYOUT_CHOICES)
    theme = models.ForeignKey(PageTheme, verbose_name=_('Theme'))
    color_scheme = models.ForeignKey(
        PageColorScheme, verbose_name=_('Color scheme'))

    class Meta:
        verbose_name = _("Page")
        verbose_name_plural = _("Pages")
        ordering = ['tree_id', 'lft']

    @cached_property
    def tree_label(self):
        '''render tree label like as `root > child > child`'''
        titles = []
        page = self
        while page:
            titles.append(page.title)
            page = page.parent
        return smart_text(' > '.join(reversed(titles)))

    @cached_property
    def own_dimensions(self):
        self_dimensions = PageDimension.objects.filter(
            page=self)
        return self_dimensions

    @cached_property
    def dimensions(self):
        # collect all dimensions
        self_dimensions = self.own_dimensions
        if self_dimensions.exists():
            return self_dimensions
        parent_dimensions = None
        if self.parent and self.parent.template_key == self.template_key:
            parent_dimensions = self.parent.dimensions
        parent_dimensions = parent_dimensions or PageDimension.objects.none()
        return parent_dimensions

    def delete(self, *args, **kwargs):
        super(Page, self).delete(*args, **kwargs)
        [d.delete() for d in self.own_dimensions]

    @cached_property
    def get_base_template(self):
        return self.theme.template

    def get_next_ordering(self, region):
        """return order for new CT in region
        """
        return len(getattr(self.content, region, [])) + 1

    @cached_property
    def get_layout_class(self):
        if self.layout == 'fluid':
            return 'container-fluid'
        else:
            return 'container'

    @cached_property
    def get_all_col_classes(self):
        STR = "col-{0}-{1}"
        return [{'col1': STR.format(d.size, d.col1_width),
                 'col2': STR.format(d.size, d.col2_width),
                 'col3': STR.format(d.size, d.col3_width)}
                for d in self.dimensions]

    def get_col_classes(self, col='col1'):
        return " ".join([
            classes.get(col)
            for classes in self.get_all_col_classes])

    @classmethod
    def register_default_processors(cls, frontend_editing=None):
        """
        Register our default request processors for the out-of-the-box
        Page experience.

        Since FeinCMS 1.11 was removed from core.

        """
        super(Page, cls).register_default_processors()

        if frontend_editing:
            cls.register_request_processor(
                edit_processors.frontendediting_request_processor,
                key='frontend_editing')
            cls.register_response_processor(
                edit_processors.frontendediting_response_processor,
                key='frontend_editing')




from django.core.exceptions import ValidationError
from django.forms import widgets
from django.forms.fields import *
from django.forms.forms import *
from django.forms.widgets import *
from horizon.forms.base import DateForm, SelfHandlingMixin
from horizon.forms.fields import (DynamicChoiceField, DynamicTypedChoiceField,
                                  IPField, IPv4, IPv6, MultiIPField,
                                  SelectWidget)
from horizon.forms.views import ModalFormMixin, ModalFormView
from horizon_contrib.forms import SelfHandlingForm
from horizon_contrib.forms import SelfHandlingModelForm
from horizon_contrib.forms.forms import SelfHandlingModelForm as SHMForm
from horizon_contrib.forms.forms import DateForm, SelfHandlingForm
from horizon_contrib.forms.models import create_or_update_and_get
from crispy_forms.bootstrap import (Accordion, AccordionGroup, InlineCheckboxes,
                                    Tab, TabHolder, FieldWithButtons, StrictButton)
from crispy_forms.helper import FormHelper
from crispy_forms.layout import (Field, Fieldset, HTML, Div,
                                 Layout, Reset, Row, Submit)
import floppyforms
from leonardo.forms.fields import *
from django_select2.forms import *

# Convenience imports for public API components.

__all__ = [
    "SelfHandlingMixin",
    "SelfHandlingForm",
    "SelfHandlingModelForm",
    "DateForm",
    "ModalFormView",
    "ModalFormMixin",
    "DynamicTypedChoiceField",
    "DynamicChoiceField",
    "IPField",
    "IPv4",
    "IPv6",
    "MultiIPField",
    "SelectWidget",

    "DateForm",
    "SelfHandlingForm",
    "SHMForm",
    "SelfHandlingModelForm",
    "create_or_update_and_get",

    # From django.forms
    "ValidationError",

    # From django.forms.fields
    'Field', 'CharField', 'IntegerField', 'DateField', 'TimeField',
    'DateTimeField', 'TimeField', 'RegexField', 'EmailField', 'FileField',
    'ImageField', 'URLField', 'BooleanField', 'NullBooleanField',
    'ChoiceField', 'MultipleChoiceField', 'ComboField', 'MultiValueField',
    'FloatField', 'DecimalField', 'SplitDateTimeField', 'IPAddressField',
    'GenericIPAddressField', 'FilePathField', 'SlugField', 'TypedChoiceField',
    'TypedMultipleChoiceField',

    # From django.forms.widgets
    "widgets",
    'Media', 'MediaDefiningClass', 'Widget', 'TextInput', 'PasswordInput',
    'HiddenInput', 'MultipleHiddenInput', 'ClearableFileInput', 'FileInput',
    'DateInput', 'DateTimeInput', 'TimeInput', 'Textarea', 'CheckboxInput',
    'Select', 'NullBooleanSelect', 'SelectMultiple', 'RadioSelect',
    'CheckboxSelectMultiple', 'MultiWidget', 'SplitDateTimeWidget',

    # From django.forms.forms
    'BaseForm', 'Form',
]



# -#- coding: utf-8 -#-

from django.db import models
from django.utils.translation import ugettext_lazy as _
from leonardo.module.web.models import Widget

from .const import ICON_CHOICES, SIZE_CHOICES


class IconWidget(Widget):

    """
    Font Awesome (the fist library) Icon Widget

    * http://fortawesome.github.io/Font-Awesome/icons/
    """

    icon = models.CharField(
        max_length=255, verbose_name=_("Icon"), choices=ICON_CHOICES)

    size = models.CharField(
        max_length=255, verbose_name=_("Size"), choices=SIZE_CHOICES, default='normal')

    spin = models.BooleanField(default=False, verbose_name=_("Spin"))

    border = models.BooleanField(default=False, verbose_name=_("Border"))

    class Meta:
        abstract = True
        verbose_name = _('Icon')
        verbose_name_plural = _('Icons')




from django.conf.urls import patterns, url, include
from .. import settings as filer_settings


urlpatterns = patterns('',
                       url(r'^' + filer_settings.FILER_PRIVATEMEDIA_STORAGE.base_url.lstrip('/'),
                           include('leonardo.module.media.server.main_server_urls')),
                       url(r'^' + filer_settings.FILER_PRIVATEMEDIA_THUMBNAIL_STORAGE.base_url.lstrip('/'),
                           include('leonardo.module.media.server.thumbnails_server_urls')),
                       )



from django.conf.urls import *

urlpatterns = patterns('leonardo.module.media.views',
    (r'^$', 'directory_list_nested', {'directory_slug': None}),
    url(r'^fullscreen/(?P<category_id>[\w\-]+)/$', 'directory_detail_standalone', name="directory_detail_standalone"),
    (r'^(?P<directory_slug>[\w\-]+)/$', 'directory_list_nested'),
    (r'^(?P<parent_directory_slug>[\w\-]+)/(?P<directory_slug>[\w\-]+)/$', 'directory_list_nested'),
    (r'^(?P<grandparent_directory_slug>[\w\-]+)/(?P<parent_directory_slug>[\w\-]+)/(?P<directory_slug>[\w\-]+)/$', 'directory_list_nested'),
)




from .downloadlist.models import DownloadListWidget
from .downloaditem.models import DownloadItemWidget
from .internetvideo.models import InternetVideoWidget
from .mediagallery.models import MediaGalleryWidget
from .pdfdocument.models import PdfDocumentWidget

from .flashobject.models import FlashObjectWidget

from .simpleimage.models import SimpleImageWidget
from .simplevideo.models import SimpleVideoWidget

from .vectorgraphics.models import VectorGraphicsWidget



# -#- coding: utf-8 -#-

from django.template import RequestContext
from django.template.loader import render_to_string
from django.utils.translation import ugettext_lazy as _
from leonardo import messages
from leonardo.forms import MultiSelectField
from leonardo.module.web.models import Widget

from .utils import add_client_type, get_client_ip

INFO_CHOICES = (
    ('browser', _('browser information')),
    ('os', _('operating system')),
    ('ip', _('IP address')),
    ('hostname', _('hostname')),
)


class ClientInfoWidget(Widget):

    show_info = MultiSelectField(
        max_length=255, default='browser', verbose_name=_("show info"), choices=INFO_CHOICES)

    class Meta:
        abstract = True
        verbose_name = _("client info")
        verbose_name_plural = _('client info')

    def render_content(self, options):
        request = options.get('request')
        uas = None
        user_agent = ''
        ip = get_client_ip(request)
        host = request.get_host()
        if 'browser' in self.show_info or 'os' in self.show_info:
            try:
                import httpagentparser
            except ImportError:
                messages.error(request, _('Please install httpagentparser'))
            else:
                uas = request.META.get('HTTP_USER_AGENT')
                user_agent = httpagentparser.detect(uas)
        request = add_client_type(request)
        context = RequestContext(request, {
            'user_agent': user_agent,
            'ip': ip,
            'host': host,
            'widget': self,
        })
        return render_to_string(self.get_template, context)





from django.conf.urls import patterns, url

from leonardo.module.search.forms import SearchForm
from haystack.views import SearchView


urlpatterns = patterns('',
                       url(r'', SearchView(form_class=SearchForm),
                           name='haystack_search'),
                       )



from haystack import indexes, site, fields
from feincms.module.page.models import Page


class PageIndex(indexes.RealTimeSearchIndex):

    """
    Index for FeinCMS Page objects

    Body is generated using a complex template which includes rendered content
    for many content objects
    """

    url = fields.CharField(model_attr="_cached_url")
    body = fields.CharField(document=True, use_template=True)

    #: Currently included so we can generate search results pages without a
    # database query (Solr will include it the results):
    title = fields.CharField(model_attr="title")

    creation_date = fields.DateTimeField(model_attr='creation_date', null=True)
    modification_date = fields.DateTimeField(model_attr='modification_date', null=True)

    def should_update(self, instance, **kwargs):
        return instance.is_active()

    def get_queryset(self):
        """Return a Django QuerySet used for all search-related queries. Currently we index all active pages"""
        return Page.objects.active()

    def get_updated_field(self):
        return "modification_date"

site.register(Page, PageIndex)




from django.conf import settings
from django.contrib import admin
from django.contrib.admin.sites import AdminSite


class LeonardoAdminSite(AdminSite):

    site_header = getattr(settings, "SITE_HEADER", "Leonardo administration")

    site_title = getattr(settings, "SITE_TITLE", "Leonardo site admin")

    def get_urls(self):

        # connect all admin members

        self._registry.update(admin.site._registry)

        return super(LeonardoAdminSite, self).get_urls()

leonardo_admin = LeonardoAdminSite(name="admin")




from django.apps import AppConfig

from .widget import *


default_app_config = 'leonardo.module.devel.Config'

LEONARDO_OPTGROUP = ('Devel widgets')


LEONARDO_WIDGETS = [
    ClientInfoWidget,
    VisualTestWidget,
]

LEONARDO_APPS = ['leonardo.module.devel']


class Config(AppConfig):
    name = 'leonardo.module.devel'
    verbose_name = "Development module"




import logging
from django.apps import AppConfig

from django.utils.translation import ugettext_lazy as _
from .widget import *


default_app_config = 'leonardo.module.web.WebConfig'

LOG = logging.getLogger(__name__)


class Default(object):

    optgroup = 'Web'

    urls_conf = 'leonardo.module.web.urls'

    @property
    def page_extensions(self):

        return [
            'feincms.module.page.extensions.excerpt',
            'feincms.module.page.extensions.relatedpages',
            'feincms.module.page.extensions.navigation',
            'feincms.module.page.extensions.sites',
            'feincms.module.page.extensions.symlinks',
            'feincms.module.page.extensions.titles',
            'leonardo.extensions.seo',
            'leonardo.extensions.datepublisher',
            'leonardo.extensions.translations',
            'leonardo.extensions.changedate',
            'leonardo.extensions.ct_tracker',
            'leonardo.extensions.featured'
            ]

    @property
    def middlewares(self):

        middlewares = [
            'leonardo.module.web.middlewares.quickstart.QuickStartMiddleware', ]

        return middlewares + [
            'leonardo.module.web.middlewares.web.WebMiddleware',
            'leonardo.module.web.middlewares.horizon.HorizonMiddleware',
        ]

    @property
    def apps(self):

        INSTALLED_APPS = []

        try:
            import sorl  # noqa
            INSTALLED_APPS += ['sorl.thumbnail']
        except Exception:
            pass

        try:
            import easy_thumbnails  # noqa
            INSTALLED_APPS += ['easy_thumbnails']
        except Exception:
            pass

        return INSTALLED_APPS + [
            'feincms',
            'mptt',
            'crispy_forms',
            'floppyforms',

            'dbtemplates',
            'leonardo.module',

            'feincms.module.page',  # noqa

            'leonardo.module.web',

            'markupfield',
        ]

    @property
    def context_processors(self):
        """return WEB Conent Type Processors
        """
        return [
            'leonardo.module.web.processors.page.add_page_if_missing',
        ]

    @property
    def widgets(self):
        return [
            ApplicationWidget,
            SiteHeadingWidget,
            MarkupTextWidget,
            FeedReaderWidget,
            HtmlTextWidget,
            PageTitleWidget,
            IconWidget,
        ]

    plugins = [
        ('leonardo.module.web.apps.horizon', _('Horizon'))
    ]

    js_files = [
        'js/lib/wow.min.js'
    ]

    angular_modules = ['ngFitText']

    css_files = [
        'css/lib/animate.css',
        'css/lib/select2.css',
    ]

    config = {
        'META_KEYWORDS': ('', _('Site specific meta keywords')),
        'META_DESCRIPTION': ('', _('Site specific meta description')),
        'META_TITLE': ('', _('Site specific meta title')),
        'DEBUG': (True, _('Debug mode')),
        'FAVICON_PATH': ('/static/img/favicon.ico', _('Favicon path')),
    }

    page_actions = ['base/page/_actions.html']
    widget_actions = ['base/widget/_actions.html']


class WebConfig(AppConfig, Default):
    name = 'leonardo.module.web'
    verbose_name = "CMS"

    def ready(self):

        # register signals
        from leonardo.module.web.signals import save as template_save
        from dbtemplates.models import Template

        Template.save = template_save


default = Default()




from django import forms
from django.contrib import admin
from django.contrib.admin.options import InlineModelAdmin, ModelAdmin
from django.contrib.contenttypes.admin import GenericTabularInline
from django.contrib.contenttypes.models import ContentType
from django.utils.translation import ugettext_lazy as _
from feincms.admin import item_editor
from feincms.module.page.modeladmins import PageAdmin as FeinPageAdmin

from .models import *


class PageDimensionAdmin(InlineModelAdmin):

    model = PageDimension


class PageColorSchemeInlineAdmin(admin.TabularInline):

    model = PageColorScheme

    extra = 1


class PageAdmin(FeinPageAdmin):

    fieldsets = [
        (None, {
            'fields': [
                ('title', 'slug'),
                ('active', 'in_navigation'),
            ],
        }),
        (_('Other options'), {
            'classes': ['collapse'],
            'fields': [
                'template_key', 'parent', 'override_url', 'redirect_to', 'theme', 'color_scheme', 'layout'],
        }),
        # <-- insertion point, extensions appear here, see insertion_index
        # above
        item_editor.FEINCMS_CONTENT_FIELDSET,
    ]

admin.site.register(Page, PageAdmin)


class PageDimensionAdmin(ModelAdmin):

    pass

admin.site.register(PageDimension, PageDimensionAdmin)


class WidgetDimensionAdmin(ModelAdmin):

    pass

admin.site.register(WidgetDimension, WidgetDimensionAdmin)


class WidgetDimensionInlineAdmin(GenericTabularInline):

    ct_field = "widget_type"
    ct_fk_field = "widget_id"

    model = WidgetDimension

    extra = 1


class PageColorSchemeAdmin(ModelAdmin):

    pass

admin.site.register(PageColorScheme, PageColorSchemeAdmin)


class WidgetContentThemeForm(forms.ModelForm):

    widget_class = forms.ChoiceField(
        choices=[],
        required=False,
    )

    class Meta:
        exclude = tuple()
        model = WidgetContentTheme

    def __init__(self, *args, **kwargs):
        super(WidgetContentThemeForm, self).__init__(*args, **kwargs)

        choices = [(t.model_class().__name__ if t.model_class() else None, t)
                   for t in ContentType.objects.filter(app_label__in=['web'])]
        self.fields['widget_class'].choices = choices


class WidgetContentThemeAdmin(ModelAdmin):

    form = WidgetContentThemeForm

    list_display = ('name', 'label', 'template', 'widget_class')

    list_filter = ('widget_class',)


admin.site.register(WidgetContentTheme, WidgetContentThemeAdmin)


class WidgetBaseThemeForm(forms.ModelForm):

    widget_class = forms.ChoiceField(
        choices=[],
        required=False,
    )

    class Meta:
        exclude = tuple()
        model = WidgetBaseTheme


class WidgetBaseThemeAdmin(ModelAdmin):

    form = WidgetContentThemeForm

    list_display = ('name', 'label', 'template')

admin.site.register(WidgetBaseTheme, WidgetBaseThemeAdmin)


class PageThemeAdmin(ModelAdmin):

    inlines = [PageColorSchemeInlineAdmin, ]

admin.site.register(PageTheme, PageThemeAdmin)



from __future__ import absolute_import

from horizon.tables.views import DataTableView  # noqa
from horizon.tables.views import MixedDataTableView  # noqa
from horizon.tables.views import MultiTableMixin  # noqa
from horizon.tables.views import MultiTableView  # noqa
from horizon_contrib.forms.views import (ContextMixin, CreateView,
                                         ModalFormView, ModelFormMixin,
                                         UpdateView)
from leonardo.apps import standalone

# set default template name
ModalFormView.template_name = 'leonardo/common/modal.html'

# mark all modal views as standalone
ModalFormView.get = standalone(ModalFormView.get)
UpdateView.get = standalone(UpdateView.get)
CreateView.get = standalone(CreateView.get)


__all__ = ('DataTableView', 'MixedDataTableView', 'MultiTableView',
           'MultiTableMixin', 'ContextMixin',
           'ModalFormView', 'UpdateView', 'CreateView',
           'ModelFormMixin')




from django.core.urlresolvers import reverse
from django.core.urlresolvers import reverse_lazy
from django.utils.translation import ugettext_lazy as _

from django.conf import settings
from leonardo import forms
from leonardo import messages

from .forms import LoginForm, SignupForm

from django.contrib.sites.models import Site

from django.contrib.auth import logout as auth_logout
from django.shortcuts import redirect

from django.views.decorators.debug import sensitive_post_parameters  # noqa


class LoginView(forms.ModalFormView):
    form_class = LoginForm
    template_name = 'leonardo_auth/login.html'
    success_url = getattr(settings, 'LOGIN_REDIRECT_URL', '/')

    redirect_field_name = "next"

    def get_success_url(self):
        # Explicitly passed ?next= URL takes precedence
        ret = self.request.GET.get(self.redirect_field_name, self.success_url)
        return ret

    def get_context_data(self, **kwargs):
        ret = super(LoginView, self).get_context_data(**kwargs)
        redirect_field_value = self.request.REQUEST \
            .get(self.redirect_field_name)
        ret.update({
            "url": self.request.build_absolute_uri(),
            "view_name": _("Login"),
            "modal_size": 'sm',
            "site": Site.objects.get_current(),
            "redirect_field_name": self.redirect_field_name,
            "redirect_field_value": redirect_field_value})
        return ret

    def get_initial(self):
        return {}


class SignupView(forms.ModalFormView):
    template_name = "leonardo/common/modal.html"
    form_class = SignupForm
    redirect_field_name = "next"
    success_url = getattr(settings, 'LOGIN_REDIRECT_URL', '/')

    def get_success_url(self):
        # Explicitly passed ?next= URL takes precedence
        ret = getattr(self.kwargs, self.redirect_field_name, self.success_url)
        return ret

    def get_context_data(self, **kwargs):
        ret = super(SignupView, self).get_context_data(**kwargs)
        redirect_field_name = self.redirect_field_name
        redirect_field_value = self.request.REQUEST.get(redirect_field_name)
        ret.update({
            "url": self.request.build_absolute_uri(),
            "view_name": _("Register"),
            "redirect_field_name": redirect_field_name,
            "redirect_field_value": redirect_field_value})
        return ret


class LogoutView(forms.ModalFormView):

    template_name = "leonrdo/common/modal.html"
    redirect_field_name = "next"

    def get(self, *args, **kwargs):
        return self.post(*args, **kwargs)

    def post(self, *args, **kwargs):
        url = self.get_redirect_url()
        if self.request.user.is_authenticated():
            self.logout()
        return redirect(url)

    def logout(self):
        messages.success(self.request, _('Logout complete'))
        auth_logout(self.request)

        self.request.session.flush()
        if hasattr(self.request, 'user'):
            from django.contrib.auth.models import AnonymousUser
            self.request.user = AnonymousUser()

    def get_context_data(self, **kwargs):
        ctx = kwargs
        redirect_field_value = self.request.REQUEST \
            .get(self.redirect_field_name)
        ctx.update({
            "redirect_field_name": self.redirect_field_name,
            "redirect_field_value": redirect_field_value})
        return ctx

    def get_redirect_url(self):
        ret = self.request.GET.get(
            self.redirect_field_name, settings.LOGOUT_URL)
        return ret



# -#- coding: utf-8 -#-

from django.db import models
from django.utils.translation import ugettext_lazy as _
from leonardo.module.media.fields import VideoField
from leonardo.module.web.models import Widget
from leonardo.module.web.widgets.forms import WidgetUpdateForm


class VideoForm(WidgetUpdateForm):

    video = VideoField()


class SimpleVideoWidget(Widget):

    feincms_item_editor_form = VideoForm

    icon = "fa fa-video-camera"

    thumb = models.ForeignKey("media.Image", verbose_name=_(
        "thumbnail"), blank=True, null=True, related_name="%(app_label)s_%(class)s_thumbnails")
    video = models.ForeignKey(
        "media.Video", verbose_name=_("video"), related_name="%(app_label)s_%(class)s_videos")
    width = models.IntegerField(verbose_name=_("width"), default=100)
    height = models.IntegerField(verbose_name=_("height"), default=100)

    class Meta:
        abstract = True
        verbose_name = _("simple video")
        verbose_name_plural = _('simple video')




from django.apps import AppConfig

from django.utils.translation import ugettext_lazy as _

from .widget import *

default_app_config = 'leonardo.module.media.MediaConfig'


class Default(object):

    optgroup = 'Media'

    @property
    def apps(self):
        return [
            'leonardo.module',
            'leonardo.module.media',
        ]

    @property
    def widgets(self):
        return [
            DownloadListWidget,
            DownloadItemWidget,
            InternetVideoWidget,
            MediaGalleryWidget,
            SimpleImageWidget,
            VectorGraphicsWidget,
            PdfDocumentWidget,
            FlashObjectWidget,
        ]

    plugins = [
        ('leonardo.module.media.apps.category_nested', 'List of directories'),
        ('leonardo.module.media.apps.category_simple', 'Simple list of directories'),
    ]

    config = {
        'MEDIA_PAGINATE_BY': (25, _('Pagination count for media files')),
        'MEDIA_PUBLIC_UPLOAD_TO': ('public', _('Prefix for public files from MEDIA_ROOT')),
        'MEDIA_PRIVATE_UPLOAD_TO': ('private', _('Prefix for private files from MEDIA_ROOT')),
        'MEDIA_IS_PUBLIC_DEFAULT': (True, _('Set uploaded files to public automatically')),
        'MEDIA_ENABLE_PERMISSIONS': (True, _(
            'Permissions for downloadable items. Experimental feature.')),
        'MEDIA_ALLOW_REGULAR_USERS_TO_ADD_ROOT_FOLDERS': (False, _('ALLOW_REGULAR_USERS_TO_ADD_ROOT_FOLDERS')),
        'MEDIA_THUMB_SMALL_GEOM': ('64x64', _('MEDIA_THUMB_SMALL_GEOM')),
        'MEDIA_THUMB_SMALL_OPT': ('', _('Another options for small thumnails')),
        'MEDIA_THUMB_MEDIUM_GEOM': ('256x256', _('MEDIA_THUMB_MEDIUM_GEOM')),
        'MEDIA_THUMB_MEDIUM_OPT': ('', _('Another options for medium thumnails')),
        'MEDIA_THUMB_LARGE_GEOM': ('768x768', _('MEDIA_THUMB_LARGE_GEOM')),
        'MEDIA_THUMB_LARGE_OPT': ('', _('Another options for large thumnails')),
        'MEDIA_LOGICAL_STRUCTURE': (False, _('If is True all folders and files will has same path in the OS')),
    }

    page_actions = ['media/_actions.html']


class MediaConfig(AppConfig, Default):
    name = 'leonardo.module.media'
    verbose_name = "Media"

default = Default()



# -#- coding: utf-8 -#-

import json

from django.db import models
from django.utils.translation import ugettext_lazy as _
from feincms.admin.item_editor import ItemEditorForm
from leonardo.module.web.models import Widget


class TableFormatter(object):

    """
    Table formatter which should convert a structure of nested lists into
    a suitable HTML table representation.
    """

    def __init__(self, **kwargs):
        for k, v in kwargs.items():
            setattr(self, k, v)

    def __call__(self, data):
        return self.format_table(data)

    def format_table(self, data):
        return u'<table class="table"><tbody>%s</tbody></table>' % u''.join(
            self.format_row(index, row) for index, row in enumerate(data))

    def format_row(self, index, row, alt=False):
        self.row_index = index
        if alt:
            css_class = 'odd'
        else:
            css_class = 'even'
        data = u''.join(self.format_cell(index, cell)
                        for index, cell in enumerate(row))
        return u'<tr class="%s">%s</tr>' % (css_class, data)

    def format_cell(self, index, cell):
        return u'<td>%s</td>' % cell


class TitleTableFormatter(TableFormatter):

    """
    TitleTableFormatter(first_row_title=True, first_column_title=True)
    """

    def format_table(self, data):
        output = ""
        for index, row in enumerate(data):
            if index == 0:
                output += '<thead>'
                output += self.format_row(index, row)
                output += '</thead><tbody>'
            else:
                if index % 2 == 0:
                    output += self.format_row(index, row)
                else:
                    output += self.format_row(index, row, True)
        return u'<table class="table">%s</tbody></table>' % output

    def format_cell(self, index, cell):
        if (not self.row_index and getattr(self, 'first_row_title', True)) or \
                (not index and getattr(self, 'first_column_title', True)):
            return u'<th>%s</th>' % cell
        return u'<td>%s</td>' % cell


class TableWidgetAdminForm(ItemEditorForm):

    def __init__(self, *args, **kwargs):
        super(TableWidgetAdminForm, self).__init__(*args, **kwargs)
        self.fields['data'].widget.attrs.update({'class': 'item-table'})


class TableWidget(Widget):

    """
    Content to edit and display HTML tables.

    This widget can be used to edit and display nicely formatted HTML tables. 
    It is easy to specify your own table renderers.
    """

    form = TableWidgetAdminForm

    feincms_item_editor_form = TableWidgetAdminForm

    feincms_item_editor_includes = {
        'head': ['admin/widget/table/init.html'],
    }

    html = models.TextField('HTML', blank=True, editable=False)

    DEFAULT_TYPES = [
        ('plain', _('plain'), TableFormatter()),
        ('titlerow', _('title row'), TitleTableFormatter(
            first_row_title=True, first_column_title=False)),
        ('titlerowcol', _('title row and column'), TitleTableFormatter(
            first_row_title=True, first_column_title=True)),
    ]

    class Meta:
        abstract = True
        verbose_name = _('table')
        verbose_name_plural = _('tables')

    @classmethod
    def initialize_type(cls, TYPES=None):
        TYPES = TYPES or cls.DEFAULT_TYPES

        cls.FORMATTERS = dict((t[0], t[2]) for t in TYPES)
        cls.TYPE_CHOICES = [(t[0], t[1]) for t in TYPES]

        cls.add_to_class('type', models.CharField(_('type'), max_length=20,
                                                  choices=cls.TYPE_CHOICES,
                                                  default=cls.TYPE_CHOICES[0][0]))

        cls.add_to_class('data', models.TextField(_('data'), blank=True))

    def save(self, *args, **kwargs):
        self.data = self.data.replace('\r', '\\r').replace(
            '\n', '\\n').replace('\t', '\\t')
        self.html = self.data and self.FORMATTERS[
            self.type](json.loads(self.data)) or u''

        super(TableWidget, self).save(*args, **kwargs)




from __future__ import absolute_import, unicode_literals

from django import template
from django.conf import settings
from django.core.urlresolvers import NoReverseMatch
from django.template import TemplateSyntaxError
from django.template.defaulttags import kwarg_re
from django.utils.encoding import smart_str
from django.utils.translation import ugettext as _
from feincms.templatetags.feincms_tags import _render_content
from feincms.templatetags.fragment_tags import (fragment, get_fragment,
                                                has_fragment)
from leonardo.module.web.widget.application.reverse import \
    app_reverse as do_app_reverse
from leonardo.module.web.widget.application.reverse import reverse_lazy

register = template.Library()

register.tag(fragment)
register.tag(get_fragment)
register.filter(has_fragment)


@register.simple_tag
def get_col_classes(page, region):
    data = page.get_col_classes(region)
    if str(data) == "":
        return 'col-md-12'
    else:
        return data


@register.simple_tag
def head_title(request):
    """
    {% head_title request %}
    """
    try:
        fragments = request._feincms_fragments
    except:
        fragments = {}

    if '_head_title' in fragments and fragments.get("_head_title"):
        return fragments.get("_head_title")
    else:
        return request.webcms_page.page_title


@register.inclusion_tag('leonardo/common/_region_tools.html',
                        takes_context=True)
def render_region_tools(context, feincms_object, region, request=None):
    """
    {% render_region_tools feincms_page "main" request %}

    skip rendering in standalone mode
    """

    if context.get('standalone', False) or not feincms_object:
        return {}

    edit = False

    if getattr(settings, 'LEONARDO_USE_PAGE_ADMIN', False):
        request = context.get('request', None)
        frontend_edit = request.COOKIES.get(
            'frontend_editing', False)
        if frontend_edit:
            edit = True

    return {
        'edit': edit,
        'feincms_object': feincms_object,
        'region': region,
        'widget_add_url': reverse_lazy(
            'widget_create',
            args=[feincms_object.id,
                  region,
                  '%s.%s' % (feincms_object._meta.app_label,
                             feincms_object.__class__.__name__)
                  ])
    }

STANDALONE_REGIONS = ['header', 'footer']


@register.simple_tag(takes_context=True)
def feincms_render_region(context, feincms_object, region, request=None):
    """
    {% feincms_render_region feincms_page "main" request %}

    Support for rendering Page without some regions especialy for modals
    this feature is driven by context variable
    """
    if not feincms_object:
        return ''

    if not context.get('standalone', False) or region in STANDALONE_REGIONS:
        return ''.join(
            _render_content(content, request=request, context=context)
            for content in getattr(feincms_object.content, region))
    return ''


class AppReverseNode(template.Node):

    def __init__(self, view_name, urlconf, args, kwargs, asvar):
        self.view_name = view_name
        self.urlconf = urlconf
        self.args = args
        self.kwargs = kwargs
        self.asvar = asvar

    def render(self, context):
        args = [arg.resolve(context) for arg in self.args]
        kwargs = dict([
            (smart_str(k, 'ascii'), v.resolve(context))
            for k, v in self.kwargs.items()])
        view_name = self.view_name.resolve(context)
        urlconf = self.urlconf.resolve(context)

        try:
            url = do_app_reverse(
                view_name, urlconf, args=args, kwargs=kwargs,
                current_app=context.current_app)
        except NoReverseMatch:
            if self.asvar is None:
                raise
            url = ''

        if self.asvar:
            context[self.asvar] = url
            return ''
        else:
            return url


@register.tag
def app_reverse(parser, token):
    """
    Returns an absolute URL for applications integrated with ApplicationContent
    The tag mostly works the same way as Django's own {% url %} tag::
        {% load leonardo_tags %}
        {% app_reverse "mymodel_detail" "myapp.urls" arg1 arg2 %}
        or
        {% load leonardo_tags %}
        {% app_reverse "mymodel_detail" "myapp.urls" name1=value1 %}
    The first argument is a path to a view. The second argument is the URLconf
    under which this app is known to the ApplicationContent. The second
    argument may also be a request object if you want to reverse an URL
    belonging to the current application content.
    Other arguments are space-separated values that will be filled in place of
    positional and keyword arguments in the URL. Don't mix positional and
    keyword arguments.
    If you want to store the URL in a variable instead of showing it right away
    you can do so too::
        {% app_reverse "mymodel_detail" "myapp.urls" arg1 arg2 as url %}
    """
    bits = token.split_contents()
    if len(bits) < 3:
        raise TemplateSyntaxError(
            "'%s' takes at least two arguments"
            " (path to a view and a urlconf)" % bits[0])
    viewname = parser.compile_filter(bits[1])
    urlconf = parser.compile_filter(bits[2])
    args = []
    kwargs = {}
    asvar = None
    bits = bits[3:]
    if len(bits) >= 2 and bits[-2] == 'as':
        asvar = bits[-1]
        bits = bits[:-2]

    if len(bits):
        for bit in bits:
            match = kwarg_re.match(bit)
            if not match:
                raise TemplateSyntaxError(
                    "Malformed arguments to app_reverse tag")
            name, value = match.groups()
            if name:
                kwargs[name] = parser.compile_filter(value)
            else:
                args.append(parser.compile_filter(value))

    return AppReverseNode(viewname, urlconf, args, kwargs, asvar)


@register.tag
def url(parser, token):
    return app_reverse(parser, token)


@register.inclusion_tag('leonardo/common/_feincms_object_tools.html',
                        takes_context=True)
def feincms_object_tools(context, cls_name):
    """
    {% feincms_object_tools 'article' %}
    {% feincms_object_tools 'web.page' %}

    render add feincms object entry
    """
    if context.get('standalone', False):
        return {}
    edit = False
    if getattr(settings, 'LEONARDO_USE_PAGE_ADMIN', False):
        request = context.get('request', None)
        frontend_edit = request.COOKIES.get(
            'frontend_editing', False)
        if frontend_edit:
            edit = True

    return {
        'edit': edit,
        'add_entry_url': reverse_lazy(
            'horizon:contrib:forms:create',
            args=[cls_name])
    }



'''
    Original File Adapted from http://pub.mowser.com/wiki/Main/CodeExamples
    With a few additions by Moof
    http://minidetector.googlecode.com/svn/trunk/minidetector/search_strings.txt
'''

simple_devices = [
    'sony',
    'symbian',
    'nokia',
    'samsung',
    'mobile',
    'windows ce',
    'epoc',
    'opera mini',
    'nitro',
    'j2me',
    'midp',
    'cldc',
    'netfront',
    'mot',
    'up.browser',
    'up.link',
    'audiovox',
    'blackberry',
    'ericsson',
    'panasonic',
    'philips',
    'sanyo',
    'sharp',
    'portalmmm',
    'blazer',
    'avantgo',
    'danger',
    'palm',
    'series60',
    'palmsource',
    'pocketpc',
    'smartphone',
    'rover',
    'ipaq',
    'au-mic',
    'alcatel',
    'ericy',
    'up.link',
    'docomo',
    'vodafone',
    'wap1',
    'wap2',
    'plucker',
    'sec',
    'fennec',
    'google wireless transcoder',
]


import os

HAYSTACK_CONNECTIONS = {
    'default': {
        'ENGINE': 'haystack.backends.whoosh_backend.WhooshEngine',
        'PATH': os.path.join(os.path.dirname(__file__), 'whoosh_index'),
    },
}




from __future__ import absolute_import, unicode_literals

from django.test import TestCase
from django.conf import settings
from leonardo.base import leonardo
from leonardo.module import web


class SettingsBaseTest(TestCase):

    def test_01_simple_settings_key(self):
        self.assertEqual(settings.APP_SPECIFIC_SETTINGS, True)

    def test_02_custom_apps_with_defaults(self):
        contains = False
        if 'leonardo.module.media' in settings.INSTALLED_APPS:
            contains = True
        self.assertEqual(contains, True)

    def test_get_app_modules(self):
        self.assertIn(web, leonardo.modules)

    def test_live_config(self):
        from constance import config
        self.assertEqual(
            getattr(config, 'GOOGLE_ANALYTICS_SITE_SPEED'), False)



from __future__ import unicode_literals

import codecs
import os
from collections import OrderedDict

from dbtemplates.conf import settings
from django.contrib.staticfiles.finders import get_finders
from django.contrib.staticfiles.storage import staticfiles_storage
from django.core.files.storage import FileSystemStorage
from django.core.management.base import BaseCommand, CommandError
from django.core.management.color import no_style
from django.utils.encoding import smart_text
from leonardo.utils.settings import merge
from leonardo.module.web.models import PageColorScheme, PageTheme
from ._utils import app_template_dirs, DIRS, get_or_create_template

from django.contrib.staticfiles.management.commands.collectstatic \
    import Command as CollectStatic


class Command(BaseCommand):

    """
    Command that allows to copy or symlink static files from different
    locations to the settings.STATIC_ROOT.
    """
    help = "Collect static files in a single location."
    requires_system_checks = False

    def __init__(self, *args, **kwargs):
        super(Command, self).__init__(*args, **kwargs)
        self.copied_files = []
        self.symlinked_files = []
        self.unmodified_files = []
        self.post_processed_files = []
        self.storage = staticfiles_storage
        self.style = no_style()
        try:
            self.storage.path('')
        except NotImplementedError:
            self.local = False
        else:
            self.local = True

    def add_arguments(self, parser):
        parser.add_argument('--noinput',
                            action='store_false', dest='interactive', default=True,
                            help="Do NOT prompt the user for input of any kind.")
        parser.add_argument('--no-post-process',
                            action='store_false', dest='post_process', default=True,
                            help="Do NOT post process collected files.")
        parser.add_argument('-i', '--ignore', action='append', default=[],
                            dest='ignore_patterns', metavar='PATTERN',
                            help="Ignore files or directories matching this glob-style "
                            "pattern. Use multiple times to ignore more.")
        parser.add_argument('-n', '--dry-run',
                            action='store_true', dest='dry_run', default=False,
                            help="Do everything except modify the filesystem.")
        parser.add_argument('-c', '--clear',
                            action='store_true', dest='clear', default=False,
                            help="Clear the existing files using the storage "
                            "before trying to copy or link the original file.")
        parser.add_argument('-l', '--link',
                            action='store_true', dest='link', default=False,
                            help="Create a symbolic link to each file instead of copying.")
        parser.add_argument('--no-default-ignore', action='store_false',
                            dest='use_default_ignore_patterns', default=True,
                            help="Don't ignore the common private glob-style patterns 'CVS', "
                            "'.*' and '*~'.")

    def set_options(self, **options):
        """
        Set instance variables based on an options dict
        """
        self.interactive = False
        self.verbosity = options['verbosity']
        self.symlink = ""
        self.clear = False
        ignore_patterns = []
        self.ignore_patterns = list(set(ignore_patterns))
        self.page_themes_updated = 0
        self.skins_updated = 0

    def collect(self):
        """
        Load and save ``PageColorScheme`` for every ``PageTheme``

        .. code-block:: bash

            static/themes/bootswatch/united/variables.scss
            static/themes/bootswatch/united/styles.scss

        """

        self.ignore_patterns = [
            '*.png', '*.jpg', '*.js', '*.gif', '*.ttf', '*.md', '*.rst',
            '*.svg']
        page_themes = PageTheme.objects.all()

        for finder in get_finders():
            for path, storage in finder.list(self.ignore_patterns):
                for t in page_themes:
                    static_path = 'themes/{0}'.format(t.name.split('/')[-1])
                    if static_path in path:
                        try:
                            page_theme = PageTheme.objects.get(id=t.id)
                        except PageTheme.DoesNotExist:
                            raise Exception("Run sync_themes before this command")
                        except Exception as e:
                            self.stdout.write(
                                "Cannot load {} into database original error: {}".format(t, e))

                        # find and load skins
                        skins_path = os.path.join(
                            storage.path('/'.join(path.split('/')[0:-1])))
                        for dirpath, skins, filenames in os.walk(skins_path):
                            for skin in [s for s in skins if s not in ['fonts']]:
                                for skin_dirpath, skins, filenames in os.walk(os.path.join(dirpath, skin)):
                                    skin, created = PageColorScheme.objects.get_or_create(
                                        theme=page_theme, label=skin, name=skin.title())
                                    for f in filenames:
                                        if 'styles' in f:
                                            with codecs.open(os.path.join(skin_dirpath, f)) as style_file:
                                                skin.styles = style_file.read()
                                        elif 'variables' in f:
                                            with codecs.open(os.path.join(skin_dirpath, f)) as variables_file:
                                                skin.variables = variables_file.read()
                                    skin.save()
                                    self.skins_updated += 1

        self.page_themes_updated += len(page_themes)

    def handle(self, **options):
        self.set_options(**options)
        force = options.get('force', False)

        # base
        page_themes = 0

        tpl_dirs = merge(DIRS, app_template_dirs)
        templatedirs = [d for d in tpl_dirs if os.path.isdir(d)]

        for templatedir in templatedirs:
            for dirpath, subdirs, filenames in os.walk(templatedir):
                if 'base/page' in dirpath:
                    for f in filenames:
                        # ignore private and hidden members
                        if not f.startswith("_") and not f.startswith("."):
                            page_template = get_or_create_template(
                                f, force=force, prefix="base/page")
                            if not page_template:
                                self.stdout.write("Missing template %s" % f)
                                continue
                            # create themes with bases
                            try:
                                page_theme = PageTheme.objects.get(
                                    name=f.split(".")[0])
                            except PageTheme.DoesNotExist:
                                page_theme = PageTheme()
                                page_theme.label = '{} layout'.format(
                                    f.split(".")[0].title())
                                page_theme.name = f.split(".")[0]
                                page_theme.template = page_template
                                page_theme.save()
                                page_themes += 1

        self.stdout.write(
            'Successfully synced {} page themes'.format(page_themes))

        message = ['\n']

        if self.is_local_storage() and self.storage.location:
            destination_path = self.storage.location
            message.append(':\n\n    %s\n\n' % destination_path)
        else:
            destination_path = None
            message.append('.\n\n')

        cmd = CollectStatic()
        cmd.stdout = self.stdout
        collect_static = cmd.handle(
            **{'interactive': False,
                'link': False,
                'clear': False,
                'dry_run': False,
                'ignore_patterns': [],
                'use_default_ignore_patterns': True,
                'post_process': True,
                'verbosity': 1})

        collected = self.collect()

        self.stdout.write("Page themes synced {}".format(self.page_themes_updated))
        self.stdout.write("Page Skins synced {}".format(self.skins_updated))

    def log(self, msg, level=2):
        """
        Small log helper
        """
        if self.verbosity >= level:
            self.stdout.write(msg)

    def is_local_storage(self):
        return isinstance(self.storage, FileSystemStorage)

    def clear_dir(self, path):
        """
        Deletes the given relative path using the destination storage backend.
        """
        if not self.storage.exists(path):
            return

        dirs, files = self.storage.listdir(path)
        for f in files:
            fpath = os.path.join(path, f)
            if self.dry_run:
                self.log("Pretending to delete '%s'" %
                         smart_text(fpath), level=1)
            else:
                self.log("Deleting '%s'" % smart_text(fpath), level=1)
                self.storage.delete(fpath)
        for d in dirs:
            self.clear_dir(os.path.join(path, d))

    def delete_file(self, path, prefixed_path, source_storage):
        """
        Checks if the target file should be deleted if it already exists
        """
        if self.storage.exists(prefixed_path):
            try:
                # When was the target file modified last time?
                target_last_modified = \
                    self.storage.modified_time(prefixed_path)
            except (OSError, NotImplementedError, AttributeError):
                # The storage doesn't support ``modified_time`` or failed
                pass
            else:
                try:
                    # When was the source file modified last time?
                    source_last_modified = source_storage.modified_time(path)
                except (OSError, NotImplementedError, AttributeError):
                    pass
                else:
                    # The full path of the target file
                    if self.local:
                        full_path = self.storage.path(prefixed_path)
                    else:
                        full_path = None
                    # Skip the file if the source file is younger
                    # Avoid sub-second precision (see #14665, #19540)
                    if (target_last_modified.replace(microsecond=0)
                            >= source_last_modified.replace(microsecond=0)):
                        if not ((self.symlink and full_path
                                 and not os.path.islink(full_path)) or
                                (not self.symlink and full_path
                                 and os.path.islink(full_path))):
                            if prefixed_path not in self.unmodified_files:
                                self.unmodified_files.append(prefixed_path)
                            self.log("Skipping '%s' (not modified)" % path)
                            return False
            # Then delete the existing file if really needed
            if self.dry_run:
                self.log("Pretending to delete '%s'" % path)
            else:
                self.log("Deleting '%s'" % path)
                self.storage.delete(prefixed_path)
        return True

    def link_file(self, path, prefixed_path, source_storage):
        """
        Attempt to link ``path``
        """
        # Skip this file if it was already copied earlier
        if prefixed_path in self.symlinked_files:
            return self.log("Skipping '%s' (already linked earlier)" % path)
        # Delete the target file if needed or break
        if not self.delete_file(path, prefixed_path, source_storage):
            return
        # The full path of the source file
        source_path = source_storage.path(path)
        # Finally link the file
        if self.dry_run:
            self.log("Pretending to link '%s'" % source_path, level=1)
        else:
            self.log("Linking '%s'" % source_path, level=1)
            full_path = self.storage.path(prefixed_path)
            try:
                os.makedirs(os.path.dirname(full_path))
            except OSError:
                pass
            try:
                if os.path.lexists(full_path):
                    os.unlink(full_path)
                os.symlink(source_path, full_path)
            except AttributeError:
                import platform
                raise CommandError("Symlinking is not supported by Python %s." %
                                   platform.python_version())
            except NotImplementedError:
                import platform
                raise CommandError("Symlinking is not supported in this "
                                   "platform (%s)." % platform.platform())
            except OSError as e:
                raise CommandError(e)
        if prefixed_path not in self.symlinked_files:
            self.symlinked_files.append(prefixed_path)

    def copy_file(self, path, prefixed_path, source_storage):
        """
        Attempt to copy ``path`` with storage
        """
        # Skip this file if it was already copied earlier
        if prefixed_path in self.copied_files:
            return self.log("Skipping '%s' (already copied earlier)" % path)
        # Delete the target file if needed or break
        if not self.delete_file(path, prefixed_path, source_storage):
            return
        # The full path of the source file
        source_path = source_storage.path(path)
        # Finally start copying
        if self.dry_run:
            self.log("Pretending to copy '%s'" % source_path, level=1)
        else:
            self.log("Copying '%s'" % source_path, level=1)
            with source_storage.open(path) as source_file:
                self.storage.save(prefixed_path, source_file)
        self.copied_files.append(prefixed_path)



#-*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.core.files import File as DjangoFile
from django.core.management.base import BaseCommand, NoArgsCommand
from filer.models.filemodels import File
from leonardo.module.media.models import *
from filer.settings import FILER_IS_PUBLIC_DEFAULT
from filer.utils.compatibility import upath
from optparse import make_option
import os

MEDIA_MODELS = [Image, Document, Vector, Video]


class FileImporter(object):

    def __init__(self, * args, **kwargs):
        self.path = kwargs.get('path')
        self.base_folder = kwargs.get('base_folder')
        self.verbosity = int(kwargs.get('verbosity', 1))
        self.file_created = 0
        self.image_created = 0
        self.folder_created = 0

    def import_file(self, file_obj, folder):
        """
        Create a File or an Image into the given folder
        """
        created = False
        for cls in MEDIA_MODELS:
            if cls.matches_file_type(file_obj.name):

                obj, created = cls.objects.get_or_create(
                    original_filename=file_obj.name,
                    file=file_obj,
                    folder=folder,
                    is_public=FILER_IS_PUBLIC_DEFAULT)
                if created:
                    self.image_created += 1
        if not created:
            obj, created = File.objects.get_or_create(
                original_filename=file_obj.name,
                file=file_obj,
                folder=folder,
                is_public=FILER_IS_PUBLIC_DEFAULT)
            if created:
                self.file_created += 1
        if self.verbosity >= 2:
            print("file_created #%s / image_created #%s -- file : %s -- created : %s" % (self.file_created,
                                                                                         self.image_created,
                                                                                         obj, created))
        return obj

    def get_or_create_folder(self, folder_names):
        """
        Gets or creates a Folder based the list of folder names in hierarchical
        order (like breadcrumbs).

        get_or_create_folder(['root', 'subfolder', 'subsub folder'])

        creates the folders with correct parent relations and returns the
        'subsub folder' instance.
        """
        if not len(folder_names):
            return None
        current_parent = None
        for folder_name in folder_names:
            current_parent, created = Folder.objects.get_or_create(
                name=folder_name, parent=current_parent)
            if created:
                self.folder_created += 1
                if self.verbosity >= 2:
                    print("folder_created #%s folder : %s -- created : %s" % (self.folder_created,
                                                                              current_parent, created))
        return current_parent

    def walker(self, path=None, base_folder=None):
        """
        This method walk a directory structure and create the
        Folders and Files as they appear.
        """
        path = path or self.path
        base_folder = base_folder or self.base_folder
        # prevent trailing slashes and other inconsistencies on path.
        path = os.path.normpath(upath(path))
        if base_folder:
            base_folder = os.path.normpath(upath(base_folder))
            print("The directory structure will be imported in %s" % (base_folder,))
        if self.verbosity >= 1:
            print("Import the folders and files in %s" % (path,))
        root_folder_name = os.path.basename(path)
        for root, dirs, files in os.walk(path):
            rel_folders = root.partition(path)[2].strip(os.path.sep).split(os.path.sep)
            while '' in rel_folders:
                rel_folders.remove('')
            if base_folder:
                folder_names = base_folder.split('/') + [root_folder_name] + rel_folders
            else:
                folder_names = [root_folder_name] + rel_folders
            folder = self.get_or_create_folder(folder_names)
            for file_obj in files:
                dj_file = DjangoFile(open(os.path.join(root, file_obj)),
                                     name=file_obj)
                self.import_file(file_obj=dj_file, folder=folder)
        if self.verbosity >= 1:
            print(('folder_created #%s / file_created #%s / ' +
                   'image_created #%s') % (
                self.folder_created, self.file_created,
                self.image_created))


class Command(NoArgsCommand):

    """
    Import directory structure into the filer ::

        manage.py --path=/tmp/assets/images
        manage.py --path=/tmp/assets/news --folder=images
    """

    option_list = BaseCommand.option_list + (
        make_option('--path',
                    action='store',
                    dest='path',
                    default=False,
                    help='Import files located in the path into django-filer'),
        make_option('--folder',
                    action='store',
                    dest='base_folder',
                    default=False,
                    help='Specify the destination folder in which the directory structure should be imported'),
    )

    def handle_noargs(self, **options):
        file_importer = FileImporter(**options)
        file_importer.walker()





from __future__ import unicode_literals

from django.db import models
from django.utils.translation import ugettext_lazy as _

ORIENTATION_CHOICES = (
    ('vertical', 'Vertical'),
    ('horizon', 'Horizon'),
)


class NavigationWidgetMixin(models.Model):

    '''Common fields for Page navigation'''

    display_active = models.NullBooleanField(
        verbose_name=_('Display active'), default=True)

    display_in_nav = models.NullBooleanField(
        verbose_name=_('Display in navigation'), default=True)

    orientation = models.CharField(
        verbose_name=_("Orientation"), max_length=20,
        choices=ORIENTATION_CHOICES, default='horizontal')

    def _filter_active(self, items):
        if self.display_active is None:
            return items
        return [item for item in items if item.active == self.display_active]

    def _filter_in_nav(self, items):
        if self.display_in_nav is None:
            return items
        return [item for item in items if item.in_navigation == self.display_in_nav]

    def filter_items(self, items):
        '''perform filtering items by specific criteria'''
        items = self._filter_active(items)
        items = self._filter_in_nav(items)
        return items

    class Meta:
        abstract = True



from __future__ import unicode_literals, with_statement

import logging

from compressor.cache import cache_set
from compressor.conf import settings
from django import template
from django.utils import six

try:
    from importlib import import_module
except:
    from django.utils.importlib import import_module

try:
    from urllib.request import url2pathname
except ImportError:
    from urllib import url2pathname

LOG = logging.getLogger(__name__)

# Some constants for nicer handling.
SOURCE_HUNK, SOURCE_FILE = 'inline', 'file'
METHOD_INPUT, METHOD_OUTPUT = 'input', 'output'

register = template.Library()

OUTPUT_FILE = 'file'
OUTPUT_INLINE = 'inline'
OUTPUT_MODES = (OUTPUT_FILE, OUTPUT_INLINE)


def compress_monkey_patch():
    """patch all compress

    we need access to variables from widget scss

    for example we have::

        /themes/bootswatch/cyborg/_variables

    but only if is cyborg active for this reasone we need
    dynamically append import to every scss file

    """
    from compressor.templatetags import compress as compress_tags
    from compressor import base as compress_base

    compress_base.Compressor.filter_input = filter_input
    compress_base.Compressor.output = output
    compress_base.Compressor.hunks = hunks

    compress_tags.CompressorMixin.render_compressed = render_compressed
    compress_tags.CompressorMixin.render_output = render_output

    from django_pyscss import compressor as pyscss_compressor

    pyscss_compressor.DjangoScssFilter.input = input


def render_compressed(self, context, kind, mode, forced=False):

    # See if it has been rendered offline
    cached_offline = self.render_offline(context, forced=forced)
    if cached_offline:
        return cached_offline

    # Take a shortcut if we really don't have anything to do
    if ((not settings.COMPRESS_ENABLED and
         not settings.COMPRESS_PRECOMPILERS) and not forced):
        return self.get_original_content(context)

    context['compressed'] = {'name': getattr(self, 'name', None)}
    compressor = self.get_compressor(context, kind)

    # Prepare the actual compressor and check cache
    cache_key, cache_content = self.render_cached(
        compressor, kind, mode, forced=forced)
    if cache_content is not None:
        return cache_content

    # call compressor output method and handle exceptions
    try:
        rendered_output = self.render_output(
            compressor, mode, forced=forced, request=context.get('request'))
        if cache_key:
            cache_set(cache_key, rendered_output)
        assert isinstance(rendered_output, six.string_types)
        return rendered_output
    except Exception:
        if settings.DEBUG or forced:
            raise

    # Or don't do anything in production
    return self.get_original_content(context)


def render_output(self, compressor, mode, forced=False, request=None):
    return compressor.output(mode, forced=forced, request=request)


def input(self, **kwargs):
    """main override which append variables import to all scss content
    """

    with_variables = None

    try:
        page = kwargs['request'].leonardo_page
    except Exception as e:
        LOG.exception(str(e))
    else:
        with_variables = """
        @import "/themes/{}/{}/_variables";
        {}
        """.format(
            page.theme.name.lower(),
            page.color_scheme.name.lower(),
            self.content)

    return self.compiler.compile_string(
        with_variables or self.content,
        filename=self.filename)


def hunks(self, forced=False, request=None):
    """
    The heart of content parsing, iterates over the
    list of split contents and looks at its kind
    to decide what to do with it. Should yield a
    bunch of precompiled and/or rendered hunks.
    """
    enabled = settings.COMPRESS_ENABLED or forced

    for kind, value, basename, elem in self.split_contents():
        precompiled = False
        attribs = self.parser.elem_attribs(elem)
        charset = attribs.get("charset", self.charset)
        options = {
            'method': METHOD_INPUT,
            'elem': elem,
            'kind': kind,
            'basename': basename,
            'charset': charset,
            'request': request,
        }

        if kind == SOURCE_FILE:
            options = dict(options, filename=value)
            value = self.get_filecontent(value, charset)

        if self.all_mimetypes:
            precompiled, value = self.precompile(value, **options)

        if enabled:
            yield self.filter(value, **options)
        else:
            if precompiled:
                yield self.handle_output(kind, value, forced=True,
                                         basename=basename)
            else:
                yield self.parser.elem_str(elem)


def output(self, mode='file', forced=False, request=None):
    """
    The general output method, override in subclass if you need to do
    any custom modification. Calls other mode specific methods or simply
    returns the content directly.
    """
    output = '\n'.join(self.filter_input(forced, request=request))

    if not output:
        return ''

    if settings.COMPRESS_ENABLED or forced:
        filtered_output = self.filter_output(output)
        return self.handle_output(mode, filtered_output, forced)

    return output


def filter_input(self, forced=False, request=None):
    """
    Passes each hunk (file or code) to the 'input' methods
    of the compressor filters.
    """
    content = []
    for hunk in self.hunks(forced, request=request):
        content.append(hunk)
    return content




from __future__ import unicode_literals

from django.contrib.auth import models as auth_models
from django.core import urlresolvers
from django.core.exceptions import ValidationError
from django.db import models
from django.db.models import Q
from django.utils.http import urlquote
from django.utils.translation import ugettext_lazy as _
from django.conf import settings
from . import mixins
from .. import settings as filer_settings
from filer.utils.compatibility import python_2_unicode_compatible

import mptt


class FolderManager(models.Manager):

    def with_bad_metadata(self):
        return self.get_query_set().filter(has_all_mandatory_data=False)


class FolderPermissionManager(models.Manager):

    """
    Theses methods are called by introspection from "has_generic_permisison" on
    the folder model.
    """

    def get_read_id_list(self, user):
        """
        Give a list of a Folders where the user has read rights or the string
        "All" if the user has all rights.
        """
        return self.__get_id_list(user, "can_read")

    def get_edit_id_list(self, user):
        return self.__get_id_list(user, "can_edit")

    def get_add_children_id_list(self, user):
        return self.__get_id_list(user, "can_add_children")

    def __get_id_list(self, user, attr):
        if user.is_superuser or not filer_settings.FILER_ENABLE_PERMISSIONS:
            return 'All'
        allow_list = set()
        deny_list = set()
        group_ids = user.groups.all().values_list('id', flat=True)
        q = Q(user=user) | Q(group__in=group_ids) | Q(everybody=True)
        perms = self.filter(q).order_by('folder__tree_id', 'folder__level',
                                        'folder__lft')
        for perm in perms:
            p = getattr(perm, attr)

            if p is None:
                # Not allow nor deny, we continue with the next permission
                continue

            if not perm.folder:
                assert perm.type == FolderPermission.ALL

                if p == FolderPermission.ALLOW:
                    allow_list.update(Folder.objects.all().values_list('id', flat=True))
                else:
                    deny_list.update(Folder.objects.all().values_list('id', flat=True))

                continue

            folder_id = perm.folder.id

            if p == FolderPermission.ALLOW:
                allow_list.add(folder_id)
            else:
                deny_list.add(folder_id)

            if perm.type == FolderPermission.CHILDREN:
                if p == FolderPermission.ALLOW:
                    allow_list.update(
                        perm.folder.get_descendants().values_list('id', flat=True))
                else:
                    deny_list.update(
                        perm.folder.get_descendants().values_list('id', flat=True))

        # Deny has precedence over allow
        return allow_list - deny_list


class Folder(models.Model, mixins.IconsMixin):

    """
    Represents a Folder that things (files) can be put into. Folders are *NOT*
    mirrored in the Filesystem and can have any unicode chars as their name.
    Other models may attach to a folder with a ForeignKey. If the related name
    ends with "_files" they will automatically be listed in the
    folder.files list along with all the other models that link to the folder
    in this way. Make sure the linked models obey the AbstractFile interface
    (Duck Type).
    """
    file_type = 'Folder'
    is_root = False
    can_have_subfolders = True
    _icon = 'plainfolder'

    parent = models.ForeignKey('self', verbose_name=('parent'), null=True, blank=True,
                               related_name="%(app_label)s_%(class)s_children")
    name = models.CharField(_('name'), max_length=255)

    owner = models.ForeignKey(getattr(settings, 'AUTH_USER_MODEL', 'auth.User'), verbose_name=('owner'),
                              related_name='filer_owned_folders',
                              null=True, blank=True)

    uploaded_at = models.DateTimeField(_('uploaded at'), auto_now_add=True)

    created_at = models.DateTimeField(_('created at'), auto_now_add=True)
    modified_at = models.DateTimeField(_('modified at'), auto_now=True)

    objects = FolderManager()

    @property
    def file_count(self):
        if not hasattr(self, '_file_count_cache'):
            self._file_count_cache = self.media_file_files.count()
        return self._file_count_cache

    @property
    def children_count(self):
        if not hasattr(self, '_children_count_cache'):
            self._children_count_cache = self.media_folder_children.count()
        return self._children_count_cache

    @property
    def item_count(self):
        return self.file_count + self.children_count

    @property
    def files(self):
        return self.media_file_files.all()

    @property
    def logical_path(self):
        """
        Gets logical path of the folder in the tree structure.
        Used to generate breadcrumbs
        """
        folder_path = []
        if self.parent:
            folder_path.extend(self.parent.get_ancestors())
            folder_path.append(self.parent)
        return folder_path

    @property
    def pretty_logical_path(self):
        return "/%s" % "/".join([f.name for f in self.logical_path + [self]])

    @property
    def quoted_logical_path(self):
        return urlquote(self.pretty_logical_path)

    def has_edit_permission(self, request):
        return self.has_generic_permission(request, 'edit')

    def has_read_permission(self, request):
        return self.has_generic_permission(request, 'read')

    def has_add_children_permission(self, request):
        return self.has_generic_permission(request, 'add_children')

    def has_generic_permission(self, request, permission_type):
        """
        Return true if the current user has permission on this
        folder. Return the string 'ALL' if the user has all rights.
        """
        user = request.user
        if not user.is_authenticated():
            return False
        elif user.is_superuser:
            return True
        elif user == self.owner:
            return True
        else:
            if not hasattr(self, "permission_cache") or\
               permission_type not in self.permission_cache or \
               request.user.pk != self.permission_cache['user'].pk:
                if not hasattr(self, "permission_cache") or request.user.pk != self.permission_cache['user'].pk:
                    self.permission_cache = {
                        'user': request.user,
                    }

                # This calls methods on the manager i.e. get_read_id_list()
                func = getattr(FolderPermission.objects,
                               "get_%s_id_list" % permission_type)
                permission = func(user)
                if permission == "All":
                    self.permission_cache[permission_type] = True
                    self.permission_cache['read'] = True
                    self.permission_cache['edit'] = True
                    self.permission_cache['add_children'] = True
                else:
                    self.permission_cache[permission_type] = self.id in permission
            return self.permission_cache[permission_type]

    def get_admin_url_path(self):
        return urlresolvers.reverse('admin:media_folder_change',
                                    args=(self.id,))

    def get_admin_directory_listing_url_path(self):
        return urlresolvers.reverse('admin:filer-directory_listing',
                                    args=(self.id,))

    def __str__(self):
        return "%s" % (self.name,)

    def contains_folder(self, folder_name):
        try:
            self.media_folder_children.get(name=folder_name)
            return True
        except Folder.DoesNotExist:
            return False

    class Meta:
        unique_together = (('parent', 'name'),)
        ordering = ('name',)
        permissions = (("can_use_folder_listing",
                        "Can use Folder listing"),)
        app_label = 'media'
        verbose_name = _("Folder")
        verbose_name_plural = _("Folders")

# MPTT registration
try:
    mptt.register(Folder)
except mptt.AlreadyRegistered:
    pass


@python_2_unicode_compatible
class FolderPermission(models.Model):
    ALL = 0
    THIS = 1
    CHILDREN = 2

    ALLOW = 1
    DENY = 0

    TYPES = (
        (ALL, _('all items')),
        (THIS, _('this item only')),
        (CHILDREN, _('this item and all children')),
    )

    PERMISIONS = (
        (ALLOW, _('allow')),
        (DENY, _('deny')),
    )

    folder = models.ForeignKey(Folder, verbose_name=('folder'), null=True, blank=True)

    type = models.SmallIntegerField(_('type'), choices=TYPES, default=ALL)
    user = models.ForeignKey(getattr(settings, 'AUTH_USER_MODEL', 'auth.User'),
                             related_name="folder_permissions",
                             verbose_name=_("user"), blank=True, null=True)
    group = models.ForeignKey(auth_models.Group,
                              related_name="folder_permissions",
                              verbose_name=_("group"), blank=True, null=True)
    everybody = models.BooleanField(_("everybody"), default=False)

    can_edit = models.SmallIntegerField(
        _("can edit"), choices=PERMISIONS, blank=True, null=True, default=None)
    can_read = models.SmallIntegerField(
        _("can read"), choices=PERMISIONS, blank=True, null=True, default=None)
    can_add_children = models.SmallIntegerField(
        _("can add children"), choices=PERMISIONS, blank=True, null=True, default=None)

    objects = FolderPermissionManager()

    def __str__(self):
        if self.folder:
            name = '%s' % self.folder
        else:
            name = 'All Folders'

        ug = []
        if self.everybody:
            ug.append('Everybody')
        else:
            if self.group:
                ug.append("Group: %s" % self.group)
            if self.user:
                ug.append("User: %s" % self.user)
        usergroup = " ".join(ug)
        perms = []
        for s in ['can_edit', 'can_read', 'can_add_children']:
            perm = getattr(self, s)
            if perm == self.ALLOW:
                perms.append(s)
            elif perm == self.DENY:
                perms.append('!%s' % s)
        perms = ', '.join(perms)
        return "Folder: '%s'->%s [%s] [%s]" % (
            name, self.get_type_display(),
            perms, usergroup)

    def clean(self):
        if self.type == self.ALL and self.folder:
            raise ValidationError('Folder cannot be selected with type "all items".')
        if self.type != self.ALL and not self.folder:
            raise ValidationError(
                'Folder has to be selected when type is not "all items".')
        if self.everybody and (self.user or self.group):
            raise ValidationError(
                'User or group cannot be selected together with "everybody".')
        if not self.user and not self.group and not self.everybody:
            raise ValidationError(
                'At least one of user, group, or "everybody" has to be selected.')

    class Meta:
        verbose_name = _('folder permission')
        verbose_name_plural = _('folder permissions')
        app_label = 'media'



#-*- coding: utf-8 -*-
from __future__ import unicode_literals

import hashlib
import os

from django.conf import settings
from django.core import urlresolvers
from django.core.files.base import ContentFile
from django.db import models
from django.utils.translation import ugettext_lazy as _
from filer.utils.compatibility import DJANGO_1_7, python_2_unicode_compatible
from polymorphic import PolymorphicManager, PolymorphicModel
from filer.utils.files import get_valid_filename
from . import mixins
from .. import settings as filer_settings
from ..fields.multistorage_file import MultiStorageFileField


class FileManager(PolymorphicManager):

    def find_all_duplicates(self):
        r = {}
        for file_obj in self.all():
            if file_obj.sha1:
                q = self.filter(sha1=file_obj.sha1)
                if len(q) > 1:
                    r[file_obj.sha1] = q
        return r

    def find_duplicates(self, file_obj):
        return [i for i in self.exclude(pk=file_obj.pk).filter(sha1=file_obj.sha1)]


@python_2_unicode_compatible
class File(PolymorphicModel, mixins.IconsMixin):
    file_type = 'File'
    _icon = "file"
    folder = models.ForeignKey('media.Folder', verbose_name=_('folder'),
                               null=True, blank=True, related_name="%(app_label)s_%(class)s_files")
    file = MultiStorageFileField(_('file'), null=True, blank=True, max_length=255)
    _file_size = models.IntegerField(_('file size'), null=True, blank=True)

    sha1 = models.CharField(_('sha1'), max_length=40, blank=True, default='')

    has_all_mandatory_data = models.BooleanField(
        _('has all mandatory data'), default=False, editable=False)

    original_filename = models.CharField(
        _('original filename'), max_length=255, blank=True, null=True)
    name = models.CharField(max_length=255, default="", blank=True,
                            verbose_name=_('name'))
    description = models.TextField(null=True, blank=True,
                                   verbose_name=_('description'))

    owner = models.ForeignKey(getattr(settings, 'AUTH_USER_MODEL', 'auth.User'),
                              related_name='owned_%(class)ss',
                              null=True, blank=True, verbose_name=_('owner'))

    uploaded_at = models.DateTimeField(_('uploaded at'), auto_now_add=True)
    modified_at = models.DateTimeField(_('modified at'), auto_now=True)

    is_public = models.BooleanField(
        default=filer_settings.FILER_IS_PUBLIC_DEFAULT,
        verbose_name=_('Permissions disabled'),
        help_text=_('Disable any permission checking for this ' +
                    'file. File will be publicly accessible ' +
                    'to anyone.'))

    objects = FileManager()

    @classmethod
    def matches_file_type(cls, iname, ifile=None, request=None):
        return True  # I match all files...

    def __init__(self, *args, **kwargs):
        super(File, self).__init__(*args, **kwargs)
        self._old_is_public = self.is_public

    def _move_file(self):
        """
        Move the file from src to dst.
        """
        src_file_name = self.file.name
        dst_file_name = self._meta.get_field('file').generate_filename(
            self, self.original_filename)

        if self.is_public:
            src_storage = self.file.storages['private']
            dst_storage = self.file.storages['public']
        else:
            src_storage = self.file.storages['public']
            dst_storage = self.file.storages['private']

        # delete the thumbnail
        # We are toggling the is_public to make sure that easy_thumbnails can
        # delete the thumbnails
        self.is_public = not self.is_public
        self.file.delete_thumbnails()
        self.is_public = not self.is_public
        # This is needed because most of the remote File Storage backend do not
        # open the file.
        src_file = src_storage.open(src_file_name)
        src_file.open()
        self.file = dst_storage.save(dst_file_name,
                                     ContentFile(src_file.read()))
        src_storage.delete(src_file_name)

    def _copy_file(self, destination, overwrite=False):
        """
        Copies the file to a destination files and returns it.
        """

        if overwrite:
            # If the destination file already exists default storage backend
            # does not overwrite it but generates another filename.
            # TODO: Find a way to override this behavior.
            raise NotImplementedError

        src_file_name = self.file.name
        storage = self.file.storages['public' if self.is_public else 'private']

        # This is needed because most of the remote File Storage backend do not
        # open the file.
        src_file = storage.open(src_file_name)
        src_file.open()
        return storage.save(destination, ContentFile(src_file.read()))

    def generate_sha1(self):
        sha = hashlib.sha1()
        self.file.seek(0)
        while True:
            buf = self.file.read(104857600)
            if not buf:
                break
            sha.update(buf)
        self.sha1 = sha.hexdigest()
        # to make sure later operations can read the whole file
        self.file.seek(0)

    def save(self, *args, **kwargs):
        # check if this is a subclass of "File" or not and set
        # _file_type_plugin_name
        if self.__class__ == File:
            # what should we do now?
            # maybe this has a subclass, but is being saved as a File instance
            # anyway. do we need to go check all possible subclasses?
            pass
        elif issubclass(self.__class__, File):
            self._file_type_plugin_name = self.__class__.__name__
        # cache the file size
        # TODO: only do this if needed (depending on the storage backend the whole
        # file will be downloaded)
        try:
            self._file_size = self.file.size
        except:
            pass
        if self._old_is_public != self.is_public and self.pk:
            self._move_file()
            self._old_is_public = self.is_public
        # generate SHA1 hash
        # TODO: only do this if needed (depending on the storage backend the whole
        # file will be downloaded)
        try:
            self.generate_sha1()
        except Exception:
            pass
        # experimentl feature
        if getattr(settings, 'MEDIA_LOGICAL_STRUCTURE', False):
            # relocate file to new folder
            if self.folder:
                try:
                    self.relocate_file()
                except Exception:
                    pass
        super(File, self).save(*args, **kwargs)
    save.alters_data = True

    def delete(self, *args, **kwargs):
        # Delete the model before the file
        super(File, self).delete(*args, **kwargs)
        # Delete the file if there are no other Files referencing it.
        if not File.objects.filter(file=self.file.name, is_public=self.is_public).exists():
            self.file.delete(False)
    delete.alters_data = True

    @property
    def label(self):
        if self.name in ['', None]:
            text = self.original_filename or 'unnamed file'
        else:
            text = self.name
        text = "%s" % (text,)
        return text

    def __lt__(self, other):
        return self.label.lower() < other.label.lower()

    def has_edit_permission(self, request):
        return self.has_generic_permission(request, 'edit')

    def has_read_permission(self, request):
        return self.has_generic_permission(request, 'read')

    def has_add_children_permission(self, request):
        return self.has_generic_permission(request, 'add_children')

    def has_generic_permission(self, request, permission_type):
        """
        Return true if the current user has permission on this
        image. Return the string 'ALL' if the user has all rights.
        """
        user = request.user
        if not user.is_authenticated():
            return False
        elif user.is_superuser:
            return True
        elif user == self.owner:
            return True
        elif self.folder:
            return self.folder.has_generic_permission(request, permission_type)
        else:
            return False

    def __str__(self):
        if self.name in ('', None):
            text = "%s" % (self.original_filename,)
        else:
            text = "%s" % (self.name,)
        return text

    def get_admin_url_path(self):
        if DJANGO_1_7:
            model_name = self._meta.module_name
        else:
            model_name = self._meta.model_name
        return urlresolvers.reverse(
            'admin:%s_%s_change' % (self._meta.app_label,
                                    model_name,),
            args=(self.pk,)
        )

    @property
    def url(self):
        """
        to make the model behave like a file field
        """
        try:
            r = self.file.url
        except:
            r = ''
        return r

    @property
    def path(self):
        try:
            return self.file.path
        except:
            return ""

    @property
    def size(self):
        return self._file_size or 0

    @property
    def extension(self):
        filetype = os.path.splitext(self.file.name)[1].lower()
        if len(filetype) > 0:
            filetype = filetype[1:]
        return filetype

    @property
    def get_logical_path(self):
        '''returns logical path like /directory/file.jpg'''
        folders = [f.quoted_logical_path for f in self.logical_path]
        return os.path.join('/'.join(folders),
                            get_valid_filename(self.original_filename))

    @property
    def pretty_logical_path(self):
        '''returns pretty logical path like /directory/File.jpg'''
        folders = [f.quoted_logical_path for f in self.logical_path]
        return os.path.join('/'.join(folders),
                            self.label)

    def relocate_file(self):
        '''relocate file to new directory'''
        old_path = self.file.path
        self.file.name = self.get_logical_path[1:]
        if self.file.path != old_path:
            try:
                os.makedirs(os.path.dirname(self.file.path))
            except:
                pass
            os.rename(old_path, self.file.path)

    @property
    def logical_folder(self):
        """
        if this file is not in a specific folder return the Special "unfiled"
        Folder object
        """
        if not self.folder:
            from filer.models.virtualitems import UnfiledImages
            return UnfiledImages()
        else:
            return self.folder

    @property
    def logical_path(self):
        """
        Gets logical path of the folder in the tree structure.
        Used to generate breadcrumbs
        """
        folder_path = []
        if self.folder:
            folder_path.extend(self.folder.get_ancestors())
        folder_path.append(self.logical_folder)
        return folder_path

    @property
    def duplicates(self):
        return File.objects.find_duplicates(self)

    class Meta:
        app_label = 'media'
        verbose_name = _('file')
        verbose_name_plural = _('files')



from haystack import indexes, fields
from leonardo.module.web.models import Page


class PageIndex(indexes.SearchIndex, indexes.Indexable):

    """
    Index for FeinCMS Page objects

    Body is generated using a complex template which includes rendered content
    for many content objects
    """

    url = fields.CharField(model_attr="_cached_url")
    text = fields.CharField(document=True, use_template=True)

    title = fields.CharField(model_attr="title")
    slug = fields.CharField(model_attr="slug")
    page_title = fields.CharField(model_attr="_page_title")
    meta_description = fields.CharField(model_attr="meta_description")
    meta_keywords = fields.CharField(model_attr="meta_keywords")
    content_title = fields.CharField(model_attr="_content_title")
    content = fields.CharField(model_attr="content")

    creation_date = fields.DateTimeField(model_attr='creation_date', null=True)
    modification_date = fields.DateTimeField(model_attr='modification_date', null=True)

    def should_update(self, instance, **kwargs):
        return instance.is_active()

    def get_model(self):
        return Page

    def index_queryset(self, using=None):
        """Used when the entire index for model is updated."""
        return self.get_model().objects.filter(active=True)

    def get_updated_field(self):
        return "modification_date"




from feincms.templatetags.feincms_page_tags import *


class LanguageLinksNode(SimpleAssignmentNodeWithVarAndArgs):
    """
    ::
        {% feincms_languagelinks for feincms_page as links [args] %}
    This template tag needs the translations extension.
    Arguments can be any combination of:
    * all or existing: Return all languages or only those where a translation
      exists
    * excludecurrent: Excludes the item in the current language from the list
    * request=request: The current request object, only needed if you are using
      AppContents and need to append the "extra path"
    The default behavior is to return an entry for all languages including the
    current language.
    Example::
      {% feincms_languagelinks for feincms_page as links all,excludecurrent %}
      {% for key, name, link in links %}
          <a href="{% if link %}{{ link }}{% else %}/{{ key }}/{% endif %}">
            {% trans name %}</a>
      {% endfor %}
    """

    def what(self, page, args):
        only_existing = args.get('existing', False)
        exclude_current = args.get('excludecurrent', False)

        # Preserve the trailing path when switching languages if extra_path
        # exists (this is mostly the case when we are working inside an
        # ApplicationContent-managed page subtree)
        trailing_path = ''
        request = args.get('request', None)
        if request:
            # optionaly if is there request use app extra
            try:
                # Trailing path without first slash
                trailing_path = request._feincms_extra_context.get(
                    'extra_path', '')[1:]
            except:
                pass

        translations = dict(
            (t.language, t) for t in page.available_translations())
        translations[page.language] = page

        links = []
        for key, name in settings.LANGUAGES:
            if exclude_current and key == page.language:
                continue

            # hardcoded paths... bleh
            if key in translations:
                links.append((
                    key,
                    name,
                    translations[key].get_absolute_url() + trailing_path))
            elif not only_existing:
                links.append((key, name, None))

        return links

register.tag(
    'feincms_languagelinks',
    do_simple_assignment_node_with_var_and_args_helper(LanguageLinksNode))




from collections import OrderedDict

from django.apps import apps
from django.core import serializers
from django.core.management.base import BaseCommand, CommandError
from django.db import DEFAULT_DB_ALIAS, router


class Command(BaseCommand):
    help = ("Output the contents of the database as a fixture of the given "
            "format (using each model's default manager unless --all is "
            "specified).")

    def add_arguments(self, parser):
        parser.add_argument('args', metavar='app_label[.ModelName]', nargs='*',
                            help='Restricts dumped data to the specified app_label or app_label.ModelName.')
        parser.add_argument('--format', default='xml', dest='format',
                            help='Specifies the output serialization format for fixtures.')
        parser.add_argument('--indent', default=4, dest='indent', type=int,
                            help='Specifies the indent level to use when pretty-printing output.')
        parser.add_argument('--database', action='store', dest='database',
                            default=DEFAULT_DB_ALIAS,
                            help='Nominates a specific database to dump fixtures from. '
                            'Defaults to the "default" database.')
        parser.add_argument('-e', '--exclude', dest='exclude', action='append', default=[],
                            help='An app_label or app_label.ModelName to exclude '
                            '(use multiple --exclude to exclude multiple apps/models).')
        parser.add_argument('-n', '--natural', action='store_true', dest='use_natural_keys', default=True,
                            help='Use natural keys if they are available (deprecated: use --natural-foreign instead).')
        parser.add_argument('--natural-foreign', action='store_true', dest='use_natural_foreign_keys', default=True,
                            help='Use natural foreign keys if they are available.')
        parser.add_argument('--natural-primary', action='store_true', dest='use_natural_primary_keys', default=True,
                            help='Use natural primary keys if they are available.')
        parser.add_argument('-a', '--all', action='store_true', dest='use_base_manager', default=False,
                            help="Use Django's base manager to dump all models stored in the database, "
                            "including those that would otherwise be filtered or modified by a custom manager.")
        parser.add_argument('--pks', dest='primary_keys',
                            help="Only dump objects with given primary keys. "
                            "Accepts a comma separated list of keys. "
                            "This option will only work when you specify one model.")
        parser.add_argument('-o', '--output', default=None, dest='output',
                            help='Specifies file to which the output is written.')

    def handle(self, *app_labels, **options):
        format = options.get('format', 'xml')
        indent = options.get('indent', 4)
        using = options.get('database')
        excludes = options.get('exclude', [])
        output = options.get('output')
        show_traceback = options.get('traceback')
        use_natural_keys = options.get('use_natural_keys', True)
        use_natural_foreign_keys = options.get(
            'use_natural_foreign_keys') or use_natural_keys
        use_natural_primary_keys = options.get('use_natural_primary_keys', True)
        use_base_manager = options.get('use_base_manager')
        pks = options.get('primary_keys')

        if pks:
            primary_keys = pks.split(',')
        else:
            primary_keys = []

        excluded_apps = set()
        excluded_models = set()
        for exclude in excludes:
            if '.' in exclude:
                try:
                    model = apps.get_model(exclude)
                except LookupError:
                    raise CommandError('Unknown model in excludes: %s' % exclude)
                excluded_models.add(model)
            else:
                try:
                    app_config = apps.get_app_config(exclude)
                except LookupError:
                    raise CommandError('Unknown app in excludes: %s' % exclude)
                excluded_apps.add(app_config)

        if len(app_labels) == 0:
            if primary_keys:
                raise CommandError("You can only use --pks option with one model")
            app_list = OrderedDict((app_config, None)
                                   for app_config in apps.get_app_configs()
                                   if app_config.models_module is not None and app_config not in excluded_apps)
        else:
            if len(app_labels) > 1 and primary_keys:
                raise CommandError("You can only use --pks option with one model")
            app_list = OrderedDict()
            for label in app_labels:
                try:
                    app_label, model_label = label.split('.')
                    try:
                        app_config = apps.get_app_config(app_label)
                    except LookupError:
                        raise CommandError("Unknown application: %s" % app_label)
                    if app_config.models_module is None or app_config in excluded_apps:
                        continue
                    try:
                        model = app_config.get_model(model_label)
                    except LookupError:
                        raise CommandError("Unknown model: %s.%s" %
                                           (app_label, model_label))

                    app_list_value = app_list.setdefault(app_config, [])

                    # We may have previously seen a "all-models" request for
                    # this app (no model qualifier was given). In this case
                    # there is no need adding specific models to the list.
                    if app_list_value is not None:
                        if model not in app_list_value:
                            app_list_value.append(model)
                except ValueError:
                    if primary_keys:
                        raise CommandError("You can only use --pks option with one model")
                    # This is just an app - no model qualifier
                    app_label = label
                    try:
                        app_config = apps.get_app_config(app_label)
                    except LookupError:
                        raise CommandError("Unknown application: %s" % app_label)
                    if app_config.models_module is None or app_config in excluded_apps:
                        continue
                    app_list[app_config] = None

        # Check that the serialization format exists; this is a shortcut to
        # avoid collating all the objects and _then_ failing.
        if format not in serializers.get_public_serializer_formats():
            try:
                serializers.get_serializer(format)
            except serializers.SerializerDoesNotExist:
                pass

            raise CommandError("Unknown serialization format: %s" % format)

        def get_objects():
            # Collate the objects to be serialized.
            for model in sort_dependencies(app_list.items()):
                if model in excluded_models:
                    continue
                if not model._meta.proxy and router.allow_migrate(using, model):
                    if use_base_manager:
                        objects = model._base_manager
                    else:
                        objects = model._default_manager

                    queryset = objects.using(using).order_by(model._meta.pk.name)
                    if primary_keys:
                        queryset = queryset.filter(pk__in=primary_keys)
                    for obj in queryset.iterator():
                        yield obj

        try:
            self.stdout.ending = None
            stream = open(output, 'w') if output else None
            try:
                serializers.serialize(format, get_objects(), indent=indent,
                                      use_natural_foreign_keys=use_natural_foreign_keys,
                                      use_natural_primary_keys=use_natural_primary_keys,
                                      stream=stream or self.stdout)
            finally:
                if stream:
                    stream.close()
        except Exception as e:
            if show_traceback:
                raise
            #raise CommandError("Unable to serialize database: %s" % e)


def sort_dependencies(app_list):
    """Sort a list of (app_config, models) pairs into a single list of models.
    The single list of models is sorted so that any model with a natural key
    is serialized before a normal model, and any model with a natural key
    dependency has it's dependencies serialized first.
    """
    # Process the list of models, and get the list of dependencies
    model_dependencies = []
    models = set()
    for app_config, model_list in app_list:
        if model_list is None:
            model_list = app_config.get_models()

        for model in model_list:
            models.add(model)
            # Add any explicitly defined dependencies
            if hasattr(model, 'natural_key'):
                deps = getattr(model.natural_key, 'dependencies', [])
                if deps:
                    deps = [apps.get_model(dep) for dep in deps]
            else:
                deps = []

            # Now add a dependency for any FK or M2M relation with
            # a model that defines a natural key
            for field in model._meta.fields:
                if hasattr(field.rel, 'to'):
                    rel_model = field.rel.to
                    if hasattr(rel_model, 'natural_key') and rel_model != model:
                        deps.append(rel_model)
            for field in model._meta.many_to_many:
                rel_model = field.rel.to
                if hasattr(rel_model, 'natural_key') and rel_model != model:
                    deps.append(rel_model)
            model_dependencies.append((model, deps))

    model_dependencies.reverse()
    # Now sort the models to ensure that dependencies are met. This
    # is done by repeatedly iterating over the input list of models.
    # If all the dependencies of a given model are in the final list,
    # that model is promoted to the end of the final list. This process
    # continues until the input list is empty, or we do a full iteration
    # over the input models without promoting a model to the final list.
    # If we do a full iteration without a promotion, that means there are
    # circular dependencies in the list.
    model_list = []
    while model_dependencies:
        skipped = []
        changed = False
        while model_dependencies:
            model, deps = model_dependencies.pop()

            # If all of the models in the dependency list are either already
            # on the final model list, or not on the original serialization list,
            # then we've found another model with all it's dependencies satisfied.
            found = True
            for candidate in ((d not in models or d in model_list) for d in deps):
                if not candidate:
                    found = False
            if found:
                model_list.append(model)
                changed = True
            else:
                skipped.append((model, deps))
        if not changed:
            raise CommandError("Can't resolve dependencies for %s in serialized app list." %
                ', '.join('%s.%s' % (model._meta.app_label, model._meta.object_name)
                for model, deps in sorted(skipped, key=lambda obj: obj[0].__name__))
            )
        model_dependencies = skipped

    return model_list




"""
Drop-in replacement for django.contrib.messages which handles Leonardo's
messaging needs (e.g. Celery tasks, AJAX communication, etc.).

for async messages install

.. code-block:: bash

    pip install django-async-messages

from https://github.com/codeinthehole/django-async-messages

.. code-block:: python

    >>> from leonardo import messages
    >>> barry = User.objects.get(username='barry')
    >>> messages.debug(barry, "Barry was here")
    >>> messages.info(barry, "Hi, Barry")
    >>> messages.success(barry, "Barry, your report is ready")
    >>> messages.warning(barry, "Barry, you didn't lock your session")
    >>> messages.error(barry, "You are not Barry")

or standard request

.. code-block:: python

    >>> messages.error(request, "You are not Barry")

"""

from django.contrib.messages import constants
from django.contrib.auth.models import User
from horizon.messages import add_message

try:
    from async_messages import messages
    ASYNC = True
except ImportError:
    ASYNC = False


def _get_user(request):

    if isinstance(request, User):
        return request
    if hasattr(request, 'user'):
        return request.user
    return None


def debug(request, message, extra_tags='', fail_silently=False, async=False):
    """Adds a message with the ``DEBUG`` level."""
    if ASYNC and async:
        messages.debug(_get_user(request), message)
    else:
        add_message(request, constants.DEBUG, message, extra_tags=extra_tags,
                    fail_silently=fail_silently)


def info(request, message, extra_tags='', fail_silently=False, async=False):
    """Adds a message with the ``INFO`` level."""
    if ASYNC and async:
        messages.info(_get_user(request), message)
    else:
        add_message(request, constants.INFO, message, extra_tags=extra_tags,
                    fail_silently=fail_silently)


def success(request, message, extra_tags='', fail_silently=False, async=False):
    """Adds a message with the ``SUCCESS`` level."""
    if ASYNC and async:
        messages.success(_get_user(request), message)
    else:
        add_message(request, constants.SUCCESS, message, extra_tags=extra_tags,
                    fail_silently=fail_silently)


def warning(request, message, extra_tags='', fail_silently=False, async=False):
    """Adds a message with the ``WARNING`` level."""
    if ASYNC and async:
        messages.debug(_get_user(request), message)
    else:
        add_message(request, constants.WARNING, message, extra_tags=extra_tags,
                    fail_silently=fail_silently)


def error(request, message, extra_tags='', fail_silently=False, async=False):
    """Adds a message with the ``ERROR`` level."""
    if ASYNC and async:
        messages.debug(_get_user(request), message)
    else:
        add_message(request, constants.ERROR, message, extra_tags=extra_tags,
                    fail_silently=fail_silently)




from .foldermodels import Folder, FolderPermission  # noqa
from .imagemodels import Image  # noqa
from .filemodels import File  # noqa
from .clipboardmodels import Clipboard, ClipboardItem  # noqa
from .virtualitems import *
from . import tools

from .media import *


# -#- coding: utf-8 -#-
from django.utils.translation import ugettext_lazy as _

DEFAULT_WIDTH = 12

DEFAULT_CHOICE = 0

PAGE_REGIONS_LIST = (
    'col3', 'col1', 'col2', 'header', 'footer', 'toolbar', 'preview', 'helper',)

PAGE_REGIONS = (
    ('col3', _('Content')),
    ('col1', _('Left sidebar')),
    ('col2', _('Right sidebar')),
    ('header', _('Header')),
    ('footer', _('Footer')),
    ('toolbar', _('Toolbar')),
    ('preview', _('Preview')),
    ('helper', _('Helper'))
)

col1 = ('col1', _('Left column'), 'inherited')
col2 = ('col2', _('Right column'), 'inherited')
col3 = ('col3', _('Main content'))
col3_center = ('col3', _('Center column'))
col3_left = ('col3', _('Left column'))
header = ('header', _('Header'), 'inherited')
content = ('content', _('Content'))
footer = ('footer', _('Footer'), 'inherited')
toolbar = ('toolbar', _('Toolbar'), 'inherited')
preview = ('preview', _('Preview'))
helper = ('helper', _('Helper'))

PAGE_TEMPLATES = (
    # |-------------------------------|
    # |  col1   |  col3     |  col2   |
    # | ---     | flexible  | ---     |
    # |-------------------------------|
    {
        'title': _('1 column'),
        'key': 'layout_flex',
        'path': 'layout/page.html',
        'regions': (col3, header, footer, toolbar, preview, helper),
    },
    # |-------------------------------|
    # |  col1   |  col3    |  col2    |
    # | ---     | flexible | flexible |
    # |-------------------------------|
    {
        'title': _('2 same columns'),
        'key': 'layout_flex_flex',
        'path': 'layout/page.html',
        'regions': (col3_left, col2, header, footer, toolbar, preview, helper),
    },
    # |-------------------------------|
    # |  col1   |  col3     |  col2   |
    # | ---     | flexible  | fixed   |
    # |-------------------------------|
    {
        'title': _('2 columns right'),
        'key': 'layout_flex_fixed',
        'path': 'layout/page.html',
        'regions': (col3, col2, header, footer, toolbar, preview, helper),
    },
    # |-------------------------------|
    # |  col1   |  col3     |  col2   |
    # | fixed   | flexible  | ---     |
    # |-------------------------------|
    {
        'title': _('2 columns left'),
        'key': 'layout_fixed_flex',
        'path': 'layout/page.html',
        'regions': (col3, col1, header, footer, toolbar, preview, helper),
    },
    # |-------------------------------|
    # |  col1   |  col3     |  col2   |
    # | fixed   | flexible  | fixed   |
    # |-------------------------------|
    {
        'title': _('3 columns'),
        'key': 'layout_fixed_flex_fixed',
        'path': 'layout/page.html',
        'regions': (col3, col1, col2, header, footer, toolbar, preview, helper),
    },
    # |-------------------------------|
    # |  col1   |  col3     |  col2   |
    # |  flex   | flexible  |  flex   |
    # |-------------------------------|
    {
        'title': _('3 same columns'),
        'key': 'layout_flex_flex_flex',
        'path': 'layout/page.html',
        'regions': (col1, col3_center, col2, header, footer, toolbar, preview, helper),
    },
    {
        'title': _('Dashboard'),
        'key': 'dashboard',
        'path': 'layout/dashboard.html',
        'regions': (col1, col3_center, col2, header, footer, toolbar, preview, helper),
    },
    {
        'title': _('API'),
        'key': 'api',
        'path': 'rest_framework/api.html',
        'regions': (col1, col3_center, col2, header, footer, toolbar, preview, helper),
    },
)

PAGE_LAYOUT_CHOICES = (
    ('fixed', _('Fixed')), 
    ('fluid', _('Fluid')), 
    ('boxed', _('Boxed')), 
)

WIDGET_LAYOUT_CHOICES = (
    ('inline', _('Inline')), 
    ('fixed', _('Fixed')), 
    ('fluid', _('Fluid')), 
)

WIDGET_ALIGN_CHOICES = (
    ('left', _('Left')), 
    ('center', _('Center')), 
    ('right', _('Right')), 
)

TEMPLATE_LAYOUTS = {
    'layout_flex': (0, 0, 24, 0),
    'layout_flex_flex': (0, 12, 12, 0),
    'layout_flex_fixed': (0, 6, 18, 0),
    'layout_fixed_flex': (6, 0, 18, 0),
    'layout_fixed_flex_fixed': (6, 6, 12, 0),
    'layout_flex_flex_flex': (8, 8, 8, 0),
    'layout_void': (6, 6, 6, 6),
}

DISPLAY_SIZE_CHOICES = (
    ('xs', _('Extra small')),
    ('sm', _('Small')),
    ('md', _('Medium')),
    ('lg', _('Large')),
)

COLUMN_CHOICES = (
    (DEFAULT_CHOICE,  ' — '),
    (1, _('1 col')),
    (2, _('2 cols')),
    (3, _('3 cols')),
    (4, _('4 cols')),
    (5, _('5 cols')),
    (6, _('6 cols')),
    (7, _('7 cols')),
    (8, _('8 cols')),
    (9, _('9 cols')),
    (10, _('10 cols')),
    (11, _('11 cols')),
    (DEFAULT_WIDTH, _('12 cols')),
)

ROW_CHOICES = (
    (DEFAULT_CHOICE,   ' — '),
    (1, _('1 row')),
    (2, _('2 rows')),
    (3, _('3 rows')),
    (4, _('4 rows')),
    (5, _('5 rows')),
    (6, _('6 rows')),
    (7, _('7 rows')),
    (8, _('8 rows')),
    (9, _('9 rows')),
    (10, _('10 rows')),
    (11, _('11 rows')),
    (12, _('12 rows')),
)

CLEAR_CHOICES = (
    (DEFAULT_CHOICE, _('none')),
    (1, _('break before')),
    (2, _('break after')),
)

BORDER_CHOICES = (
    (DEFAULT_CHOICE, _('no border')),
    (1, _('border')),
    (2, _('wide border')),
)

ALIGN_CHOICES = (
    (DEFAULT_CHOICE, _('auto')),
    (1, _('left')),
    (2, _('center')),
    (3, _('right')),
)

VERTICAL_ALIGN_CHOICES = (
    ('top', _('top')),
    ('middle', _('middle')),
    ('bottom', _('bottom')),
)

# this wil be on the Page model
DEFAULT_DISPLAY_OPTIONS = {
    'label': None,
    'template_name': 'default',
    'style': None,
    'size': [24, 0],
    'align': ["a", "a"],
    'padding': [0, 0, 0, 0],
    'margin': [0, 0, 0, 0],
    'visible': True,
    'border': None,
    'clear': None,
    'last': False,
}




from django.utils.translation import ugettext_lazy as _

import horizon


class AuthPanels(horizon.PanelGroup):
    slug = "auth"
    name = _("Auth")
    panels = ('auth',)


class ModuleDashboard(horizon.Dashboard):
    name = _("Modules")
    slug = "module"  # this map all to /

    panels = (AuthPanels,)
    public = True
    default_panel = 'auth'

#horizon.register(ModuleDashboard)




from __future__ import unicode_literals

import os
from optparse import make_option

from django.apps import apps
from django.core.management import call_command
from django.core.management.base import BaseCommand, NoArgsCommand


class Command(BaseCommand):

    help = "Iterate over all application and generate-compile all translations"
    option_list = NoArgsCommand.option_list + (
        make_option('--apps',
                    action='store_false', dest='interactive', default=[],
                    help="Apps for updating as array delimeted with comma."),
    )

    def handle(self, *args, **options):

        skipped_paths = []

        if len(args) > 0:
            app_names = args[0]
            if not isinstance(app_names, list):
                app_names = app_names.split(',')

                _apps = []
                for app_name in app_names:
                    try:
                        app = apps.get_app_config(app_name)
                    except Exception as e:
                        skipped_paths(app_name)
                    else:
                        _apps.append(app.path)
        else:
            _apps = apps.get_app_paths()

        for app_path in _apps:
            try:
                os.chdir(app_path)
                call_command('makemessages')
                call_command('compilemessages')
            except Exception as e:
                skipped_paths.append(app_path)
                self.stderr.write(str(e))

        if skipped_paths:
            self.stderr.write('Skipped paths: \n%s' % '\n'.join(skipped_paths))
        self.stdout.write('Successfully generated and compiled all messages')
